<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Financeiro\Entities\Boletreceber as Boletrec;
use Teknisa\Model\Financeiro\Entities\Adiantareceb;
use Teknisa\Model\Financeiro\Entities\Devolucaorec;
use Teknisa\Model\Financeiro\Entities\Ajctreceber as Ajctrec;
use Teknisa\Model\Financeiro\Entities\Tipoajreceber;
use Teknisa\Model\Financeiro\Entities\Tpcredbanc;
use Teknisa\Model\Financeiro\Entities\Movcredbanc;
use Teknisa\Model\Financeiro\Entities\Movbancario as Movbanc;
use Teknisa\Model\Geral\Entities\Contcorr;
use Teknisa\Model\Geral\Entities\Empresa;
use Teknisa\Model\Geral\Entities\Histpadr;
use Geral\Util\NovoCodigo;
use Geral\Entities\Lanccont;
use Geral\Entities\Paramgeral;
use Geral\Entities\Tipobaixarec;
use Geral\Entities\Tipoctreceber;
use Fiscal\Entities\Notafiscal;
use Financeiro\Entities\Ajbolrec;
use Financeiro\Entities\Ajctreceber;
use Financeiro\Entities\Baixarecebida;
use Financeiro\Entities\Bancopref;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Cnabrec;
use Financeiro\Entities\Emprbanc;
use Financeiro\Entities\GeralRec;
use Financeiro\Entities\Movbancario;
use Financeiro\Entities\Movfluxo;
use Financeiro\Entities\Ratctreceber;
use Financeiro\Entities\Rattpctr;
use Contabilidade\Entities\Contopectbl;
use Indeterminado\Entities\Logboletoremrec;
use Teknisa\Model\Financeiro\Entities\Contareceber;
use Financeiro\Entities\Contareceber as Contarec;
use Financeiro\Logic\ContasReceber\Boleto\AssociarBoleto\ExcecaoAssociar;
use Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoCancelamento;
use Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoCnab;
use Financeiro\Logic\ContasReceber\Boleto\Exception\ExcecaoProrrogacao;
use Financeiro\Logic\ContasReceber\Boleto\OperacoesBoleto;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;
use Teknisa\FIN\Modules\ContaReceber\Services\RemessaReceberService;
use Teknisa\FIN\Modules\ContaReceber\Services\EmissaoBoletoService;
use Teknisa\FIN\Modules\ContaReceber\Services\AjusteBoletoService;
use Teknisa\FIN\Modules\ContaReceber\Services\ParcelamentoBoletoService;
use Teknisa\FIN\Modules\ContaReceber\Controllers\EmissaoBoletoController;
use Teknisa\Libs\Util\NewCode;
use Zeedhi\Framework\Remote\cURLRequest;
use Zeedhi\Framework\Remote\HttpInterface;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Request;
use Teknisa\Libs\Util\SMTP;
use Teknisa\Libs\Util\Environment;

/**
 * Class TitulosAReceberService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class TitulosAReceberService {

    protected $entityManager;
    protected $titulosAReceberFactory;
    protected $fluxoDeCaixaService;
    protected $contabilizacaoService;
    protected $remessaReceberService;
    protected $emissaoBoletoService;
    protected $parcelamentoBoletoService;
    protected $newCode;
    protected $fileUploader;
    protected $_fileServerUrl;
    
    public $returnFileServer;

    const ID_TIP_CNAB_REC     = '0';
    const ID_CNAB_N_ENVIADO   = '00';
    const TAMANHO_NRLANBOLREC = 10;
    const PREF_NOVOCOD_BOLREC = 'BOLETRECEBER';
    const STATUS_BOLETO_PEND  = 'A';
    const BOLETO_ALTERADO     = '3';
    const PERMITE_ALT_BOLETO  = '2';
    const IDBOLRECALTER_DEF   = '1';
    const IDFORMCADBREC_DEF   = '1';
    const NUM_PARCELA_INI     = '00';
    const DESCONTADO          = 'D';
    const CARTCOBREC          = '4';
    const MOVFLUXO_REAL       = 'R';
    const EM_ABERTO           = 'A';
    const MOVFLUXO_ABERTO     = 'E';
    const BAIXA_PLANEJADA     = 'E';
    const ACRESCIMO           = 'A';

    /**
     * @param EntityManager $entityManager
     * @param TitulosAReceberFactory $boletosAReceberFactory
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     * @param ContabilizacaoService $contabilizacaoService
     * @param RemessaReceberService $remessaReceberService
     * @param EmissaoBoletoService $emissaoBoletoService
     * @param AjusteBoletoService $ajusteBoletoService
     * @param AjusteContaService $ajusteContaService
     * @param EmissaoBoletoController $emissaoBoletoController
     * @param SMTP $smtp
     * @param NewCode $newCode
     */

    public function __construct(EntityManager $entityManager, TitulosAReceberFactory $boletosAReceberFactory, FluxoDeCaixaService $fluxoDeCaixaService, ContabilizacaoService $contabilizacaoService, RemessaReceberService $remessaReceberService, EmissaoBoletoService $emissaoBoletoService, AjusteBoletoService $ajusteBoletoService, AjusteContaService $ajusteContaService, EmissaoBoletoController $emissaoBoletoController, SMTP $smtp, NewCode $newCode, ParcelamentoBoletoService $parcelamentoBoletoService, $fileUploader, $fileServerUrl,  Environment $environment)    {
        $this->entityManager             = $entityManager;
        $this->titulosAReceberFactory    = $boletosAReceberFactory;
        $this->fluxoDeCaixaService       = $fluxoDeCaixaService;
        $this->contabilizacaoService     = $contabilizacaoService;
        $this->remessaReceberService     = $remessaReceberService;
        $this->emissaoBoletoService      = $emissaoBoletoService;
        $this->ajusteBoletoService       = $ajusteBoletoService;
        $this->ajusteContaService        = $ajusteContaService;
        $this->emissaoBoletoController   = $emissaoBoletoController;
        $this->smtp                      = $smtp;
        $this->newCode                   = $newCode;
        $this->parcelamentoBoletoService = $parcelamentoBoletoService;
        $this->fileUploader              = $fileUploader;
        $this->_fileServerUrl            = $fileServerUrl;
        $this->environment               = $environment;
    }

    /**
     * Gera Boleto Receber
     * @param $nrorg
     * @param $cdfiliemit
     * @param $nrlanbolrec
     * @param $cdcliente
     * @param $cdempresa
     * @param $vrbolrec
     * @param $vrfinalbrec
     * @param $dtvenbolrec
     * @param $vrmultabrec|null
     * @param $vrmordiabrec|null
     * @param $cdreducont|null
     * @param $cdnosnumrec|null
     * @param $cdbarrarec|null
     * @param $dslinhabarrec|null
     * @param $dscontarec|null
     * @param $cdtpbaixarec|null
     ){
     * @return \Financeiro\Entities\Boletreceber|null
     * @throws \Exception
     */
    public function geraBoletoReceber(
        $nrorg, $cdfiliemit, $nrlanbolrec, $cdcliente, $cdempresa, $vrbolrec, $vrfinalbrec, $dtvenbolrec, $dsboletrec,
        $vrmultabrec = NULL, $vrmordiabrec = NULL, $cdreducont = NULL, $cdnosnumrec = NULL, $cdbarrarec = NULL,
        $dslinhabarrec = NULL, $dscontarec = NULL, $cdtpbaixarec = NULL
    ){
        try {
            if($vrmultabrec == NULL && $vrmordiabrec == NULL){
                //Calcula Multa / Mora Boleto
                $multamora = self::calcMultaMoraBolRec($nrorg, $cdempresa, $cdcliente, $vrbolrec);
                $vrmultabrec = $multamora['VRMULTABREC'];
                $vrmordiabrec = $multamora['VRMORDIABREC'];
            }else{
                //Calcula-se o restante dos valores deMulta / Mora Boleto
                $multamora = self::calcMultaMoraBolRecValores($vrbolrec, $vrmultabrec, $vrmordiabrec);
            }

            // Busca Banco Preferencial
            $bancopref = self::buscaBancoPrefBolRec($nrorg, $cdempresa, $cdcliente, $cdreducont);
            
            // Operador de Lançamento
            $cdoperinclusao = $this->environment->getCdOperador();

            // Gera Boleto
            $geraboleto = $this->titulosAReceberFactory->insereBoletoReceber(
                $nrorg, $cdfiliemit, $nrlanbolrec, $cdcliente, $cdempresa, $vrbolrec, $vrfinalbrec, $dtvenbolrec,
                $multamora['VRPEMULBREC'], $vrmultabrec , $multamora['VRPEMORBREC'], $vrmordiabrec,
                $bancopref['CDREDUCONT'], $bancopref['CDCARTCOBBOL'], $bancopref['QTDIAPROTBOL'], $bancopref['TXMENSBOLBOL'],
                $bancopref['IDIMPRBOLBOL'], $bancopref['CDINS1COBBOL'], $bancopref['CDINS2COBBOL'], $dsboletrec,
                $cdnosnumrec, $cdbarrarec, $dslinhabarrec, $dscontarec, $cdtpbaixarec, $cdoperinclusao
            );

            return $geraboleto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Calcula Multa|Mora Boleto Receber
     * @param $nrorg
     * @param $cdempresa
     * @param $cdcliente
     * @param $vrbolrec
     * @return array|null
     * @throws \Exception
     */
    public function calcMultaMoraBolRec($nrorg, $cdempresa, $cdcliente, $vrbolrec){
        try{
            // Busca Cliente
            $cliente     = $this->titulosAReceberFactory->buscaCliente($nrorg, $cdcliente);
            $vrpemorbcli = $cliente->getVrpemulbcli();
            $vrpemulbcli = $cliente->getVrpemorbcli();

            // Busca Empresa
            $empresa     = $this->titulosAReceberFactory->buscaEmpresa($nrorg, $cdempresa);
            $vrpemulempr = $empresa->getVrpemulempr();
            $vrpemorempr = $empresa->getVrpemorempr();

            // Verifica Percentual Multa / Mora Cliente
            if ($vrpemorbcli || $vrpemulbcli) {
                $vrmultabrec  = $vrbolrec * ($vrpemulbcli / 100);
                $vrmordiabrec = $vrbolrec * ($vrpemorbcli / 100);
                $vrpemorbrec  = $vrpemorbcli;
                $vrpemulbrec  = $vrpemulbcli;
            } else {
                $vrmultabrec  = $vrbolrec * ($vrpemulempr / 100);
                $vrmordiabrec = $vrbolrec * ($vrpemorempr / 100);
                $vrpemorbrec  = $vrpemorempr;
                $vrpemulbrec  = $vrpemulempr;
            }

            return $multamora = [
                'VRMULTABREC'  => $vrmultabrec,
                'VRMORDIABREC' => $vrmordiabrec,
                'VRPEMORBREC'  => $vrpemorbrec,
                'VRPEMULBREC'  => $vrpemulbrec,
            ];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Calcula Multa|Mora Boleto Receber Valores
     * @param $vrbolrec
     * @param $vrmultabrec
     * @param $vrmordiabrec
     * @return array
     */
    public function calcMultaMoraBolRecValores($vrbolrec, $vrmultabrec, $vrmordiabrec){
        return [
            'VRPEMORBREC'  => ($vrmordiabrec * 100) / $vrbolrec,
            'VRPEMULBREC'  => ($vrmultabrec * 100) / $vrbolrec
        ];
    }

    /**
     * Busca Banco Preferencial Boleto Receber
     * @param $nrorg
     * @param $cdempresa
     * @param $cdcliente
     * @param $cdreducont|null
     * @return array|null
     * @throws \Exception
     */
    public function buscaBancoPrefBolRec($nrorg, $cdempresa, $cdcliente, $cdreducont = NULL){
        try{
            // Busca Banco Preferencial
            /** @var Bancopref $bancoprefer    */

            $bancoprefer = $this->titulosAReceberFactory->buscaBancoPrefDadosBoleto($nrorg, $cdcliente, $cdempresa, $cdreducont);

            // Verifica Existencia Banco Preferencial
            if($bancoprefer){
                $cdreducont   = $bancoprefer['CDREDUCONT'];
                $cdcartcobbol = $bancoprefer['CDCARTCOBR'];
                $qtdiaprotbol = $bancoprefer['QTDIAPROTEST'];
                $txmensbolbol = $bancoprefer['TXMENSBOLETO'];

                //Busca Banco Empresa
                /** @var Emprbanc $emprbanc */
                $emprbanc = Emprbanc::getRepository()->getEmprBancDadosBoleto($nrorg, $cdreducont, $cdempresa);

                $idimprbolbol = $emprbanc['IDIMPRBOLETO']?:'B';
                $cdins1cobbol = $bancoprefer['CDINS1COBRCL']?:$emprbanc['CDINS1COBR'];
                $cdins2cobbol = $bancoprefer['CDINS2COBRCL']?:$emprbanc['CDINS2COBR'];
                $cdins2cobbol = $cdins1cobbol != $cdins2cobbol?$cdins2cobbol:null;

                return $bancopref = [
                    'CDREDUCONT'   => $cdreducont,
                    'CDCARTCOBBOL' => $cdcartcobbol,
                    'QTDIAPROTBOL' => $qtdiaprotbol,
                    'TXMENSBOLBOL' => $txmensbolbol,
                    'IDIMPRBOLBOL' => $idimprbolbol,
                    'CDINS1COBBOL' => $cdins1cobbol,
                    'CDINS2COBBOL' => $cdins2cobbol
                ];
            }else return null;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera Rateio Receber
     * @param $cdfiliemit
     * @param $nrorg
     * @param $nrlanctonf
     * @param $cdfilial
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdtpctreceb
     * @param $vrorigrec
     * @return \Financeiro\Entities\Ratctreceber
     * @throws \Exception
     */
    public function geraRateioRec(
        $nrorg, $cdempresa, $cdfilial, $cdfiliemit, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdtpctreceb, $vrorigrec
    ){
        try{
            // Variável de Inicialização
            $ratctreceber = null;

            // Rateios Padrão Conta Receber
            /** @var Rattpctr $rattpctr */
            $rattpctr = $this->titulosAReceberFactory->buscaRatTpCtR($nrorg, $cdtpctreceb);

            // Verifica Não Existência Rateio Padrão Conta Receber
            if(!$rattpctr){
                // Centro de Custo Filial
                $centcustfil = $this->titulosAReceberFactory->buscaCentCustFili($cdfiliemit, $cdempresa);

                // Verifica Existência Centro de Custo Filial
                if($centcustfil) {
                    if ($centcustfil[0]['CDCENTCUST']){
                        // Insere Rateio Tipo Conta Receber[Temporário]
                        $rattpctr[0] = $this->titulosAReceberFactory->insereRatTpCtR(
                            $nrorg, $cdfilial, $cdfiliemit, $centcustfil[0]['CDCENTCUST'], 100
                        );
                    }
                }
            }

            // Verifica Existência Rateio Padrão Conta Receber
            if($rattpctr){
                // Variáveis de Inicialização
                $vrtotratcnt  = 0;
                $vrratccctrec = 0;
                $chave        = 0;

                // Percorre Rateios Padrões Conta Receber
                /** @var Rattpctr $rateio */
                foreach($rattpctr as $chave => $rateio){
                    // Verifica Filial de Rateio Conta Receber
                    $cdfiratctrec = $rateio->getCdfilial()?$rateio->getCdfirattpctr():$cdfiliemit;
                    //Vd_CentCustFil($rateio->getCdcentcust(), $cdfiratctrec);

                    // Calcula Valor Rateio Conta Receber
                    $vrratccctrec = round((round($vrorigrec, 2) * round($rateio->getVrperattpctr(), 2))/100, 2);

                    // Insere Rateio Conta Receber
                    $ratctreceber[$chave] = $this->titulosAReceberFactory->insereRatCtReceber(
                        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $rateio->getCdcentcust(), $cdfiratctrec, $vrratccctrec
                    );

                    // Soma Valor Total Rateio Conta Receber
                    $vrtotratcnt = round($vrtotratcnt, 2) + $vrratccctrec;
                }

                // Verifica Existência de Diferença do Valor Conta/Rateio
                if (round($vrtotratcnt, 2) <> round($vrorigrec, 2)){
                    // Calcula Valor Ajustado Última Conta Receber
                    $vrratccctrec = round($vrratccctrec, 2) + round($vrorigrec, 2) - round($vrtotratcnt, 2);

                    // Atualiza Último Rateio Conta Receber
                    $ratctreceber = $this->titulosAReceberFactory->atualizaRatCtReceber(
                        $ratctreceber[$chave], $vrratccctrec
                    );
                }
            }

            return $ratctreceber;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera Títulos a Receber
     * @param $row
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTituloAReceber($row, $response, $permiteParcela = true){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::geraTituloARec($row, $response, $permiteParcela);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Títulos a Receber
     * @param $row
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTituloARec($row, $response, $permiteParcela = true){
        try {
            $nrorg = $row['NRORG']; $cdfilial = $row['CDFILIAL'];
            $cdempresa = $row['CDEMPRESA']; $cdinscesta = $row['CDINSCESTA']; $cdinscmuni = $row['CDINSCMUNI'];
            $cdfiliemit = $row['CDFILIEMIT']; $idtpijurnf = $row['IDTPIJURNF']; $nrinsjurnf = $row['NRINSJURNF'];
            $cdcliente = $row['CDCLIENTE']; $idbloqueado = $row['IDBLOQUEADO']; $iddocfiscal = $row['IDDOCFISCAL'];
            $iddocfinanc = $row['IDDOCFINANC']; $vrprodutos = $row['VRPRODUTOS']; $vrnotafisc = $row['VRNOTAFISC'];
            $nrnotafisc = $row['NRNOTAFISC']; $dtemissao = $row['DTEMISSAO']; $dtentrsaid = $row['DTENTRSAID'];
            $identrsaidop = $row['IDENTRSAIDOP']; $cdtipooper = $row['CDTIPOOPER']; $cdserie = $row['CDSERIE'];
            $iddoccontab = $row['IDDOCCONTAB']; $idstatus = $row['IDSTATUS']; $iddocfiscent = $row['IDDOCFISCENT'];
            $iddocfiscsai = $row['IDDOCFISCSAI']; $iddocestoque = $row['IDDOCESTOQUE']; $idsistema = $row['IDSISTEMA'];
            $nrlicencauso = $row['NRLICENCAUSO']; $cdvendedor = $row['CDVENDEDOR']; $dtlancamennf = $row['DTLANCAMENNF'];
            $nrlanbolrec = $row['NRLANBOLREC']; $vrbolrec = $row['VRBOLREC']; $vrfinalbrec = $row['VRFINALBREC'];
            $dtvenbolrec = $row['DTVENBOLREC']; $cdclassfina = $row['CDCLASSFINA']; $dtmovfluxo = $row['DTMOVFLUXO'];
            $idmovfluxo = $row['IDMOVFLUXO']; $nrseqvenrec = $row['NRSEQVENREC']; $nrparcelarec = $row['NRPARCELAREC'];
            $dtorigvenrec = $row['DTORIGVENREC']; $vratuarec = $row['VRATUAREC']; $cdtpctreceb = $row['CDTPCTRECEB'];
            $dtatuavenrec = $row['DTATUAVENREC']; $vrorigrec = $row['VRORIGREC']; $cdfilibolrec = $row['CDFILIBOLREC'];
            $idstatrec = $row['IDSTATREC']; $qtdiasvenrec = $row['QTDIASVENREC']; $cdfimovflrec = $row['CDFIMOVFLREC'];
            $dscontarec = $row['DSCONTAREC']; $dsboletrec = $row['DSBOLETREC'];
            $vrmultabrec = (isset($row['VRMULTABREC'])) ? $row['VRMULTABREC'] : NULL;
            $vrmordiabrec = (isset($row['VRMORDIABREC'])) ? $row['VRMORDIABREC'] : NULL;
            $cdreducont = (isset($row['CDREDUCONT'])) ? $row['CDREDUCONT'] : NULL;
            $cdnosnumrec = (isset($row['CDNOSNUMREC'])) ? $row['CDNOSNUMREC'] : NULL;
            $cdbarrarec = (isset($row['CDBARRAREC'])) ? $row['CDBARRAREC'] : NULL;
            $dslinhabarrec = (isset($row['DSLINHABARREC'])) ? $row['DSLINHABARREC'] : NULL;
            $cdtpbaixarec = (isset($row['CDTPBAIXAREC'])) ? $row['CDTPBAIXAREC'] : NULL;
            $nrlanctonf = (isset($row['NRLANCTONF'])) ? $row['NRLANCTONF'] : NULL;

            // Busca Parcelas Titulo a Receber
            $parcela = $this->titulosAReceberFactory->buscaParcela($cdfilial, $nrnotafisc, $cdserie, $nrinsjurnf, $vrnotafisc, $dtemissao->format("d/m/Y H:i:s"), $nrorg);

            if($permiteParcela && $parcela){
                throw new \Exception("Existe mais de um documento com mesmo cliente, número da nota, série e data de emissão"); // Planilha de importação não possui campo para valor de parcela, só possui para valor total.
            }else{
                // Verifica Existência Lançamento NF e Gera Novo Código
                if(!$nrlanctonf)
                    $nrlanctonf = $this->geraNovoCodigo($nrorg, 'LANCNF/FILIAL'.$cdfilial);

                // Busca Nota Fiscal e Cadastra NF
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAReceberFactory->buscaNotaFiscalReceber($nrorg, $cdfilial, $nrlanctonf);
                if(!$notafiscal){
                    $notafiscal = $this->titulosAReceberFactory->insereNotaFiscalReceber(
                        $nrorg, $cdfilial, $nrlanctonf, $cdempresa, $cdinscesta, $cdinscmuni, $cdfiliemit,
                        $idtpijurnf, $nrinsjurnf, $cdcliente, $idbloqueado, $iddocfiscal, $iddocfinanc,
                        $vrprodutos, $vrnotafisc, $nrnotafisc, $dtemissao, $dtentrsaid, $identrsaidop,
                        $cdtipooper, $cdserie, $iddoccontab, $idstatus, $iddocfiscent,
                        $iddocfiscsai, $iddocestoque, $idsistema, $nrlicencauso, $cdvendedor, $dtlancamennf
                    );

                    $this->entityManager->persist($notafiscal);
                }
            }

            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $cdempresa, $cdfiliemit, $cdclassfina, $dtatuavenrec, $vratuarec, $idmovfluxo, 'E'
            );
            $this->entityManager->persist($movfluxo);

            // Gera Boleto Receber
            /** @var Boletreceber $boletreceber */
            $boletreceber = $this->geraBoletoReceber(
                $nrorg, $cdfiliemit, $nrlanbolrec, $cdcliente, $cdempresa, $vrbolrec, $vrfinalbrec, $dtvenbolrec, $dsboletrec,
                $vrmultabrec, $vrmordiabrec, $cdreducont, $cdnosnumrec, $cdbarrarec,
                $dslinhabarrec, $cdtpbaixarec
            );
            $this->entityManager->persist($boletreceber);
            $this->entityManager->flush();

            // Gera Conta Receber
            /** @var Contareceber $contareceber */
            $contareceber = $this->titulosAReceberFactory->insereContaReceber(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $dtorigvenrec,
                $vratuarec, $cdtpctreceb, $dtatuavenrec, $vrorigrec, $cdfilibolrec, $nrlanbolrec,
                $idstatrec, $qtdiasvenrec, $cdclassfina, $cdempresa, $dtmovfluxo,
                $idmovfluxo, $cdfimovflrec, $dscontarec
            );
            $this->entityManager->persist($contareceber);
            $this->entityManager->flush();

            // Verifica Status da Nota Fiscal
            if($idstatus != 'V') {
                // Gera Rateio Conta Receber
                /** @var Ratctreceber $ratctreceber */
                $ratctreceber = $this->geraRateioRec(
                    $nrorg, $cdempresa, $cdfilial, $cdfiliemit, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdtpctreceb, $vrorigrec, $cdfiliemit
                );
                
                if($ratctreceber)
                    foreach ($ratctreceber as $rateio)
                        $this->entityManager->persist($rateio);

                // Gera Contabilização Conta Receber
                $this->contabilizacaoService->contabilizaRecebimento(
                    $cdfilial, $cdempresa, $nrlanctonf, $nrparcelarec, $nrseqvenrec, $idstatrec, $cdtpctreceb
                );
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Gera Titulos a Receber
     * @param $rows
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTitulosAReceber($rows, $response, $permiteParcela = true){

        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach ($rows as $row) {
                self::geraTituloARec($row, $response, $permiteParcela);
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Titulos Renegociados
     * @param $rows
     * @param $renegociacao
     * @param $titulosReneg
     * @param $titulosMarca
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraRenegociacaoTitulos($rows, $renegociacao, $titulosReneg, $titulosMarca, $response){

        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{

            // Gera Titulos Renegociados
            foreach ($rows as $row) {
                self::geraTituloARec($row, $response);
            }

            // Insere Ajuste
            self::geraRenegociacaoAjuste($renegociacao);

            // Gera Historico de Renegociação
            $nrseqhistrenego = self::geraRenegociacaoHistorico($renegociacao);

            // Atualiza Titulo Renegociados
            self::atualizaTitulosRenegociados($rows, $renegociacao, $titulosReneg, $titulosMarca, $nrseqhistrenego, $response);

            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Ajuste de Renegociacao
     * @param $renegociacao
     * @throws \Exception
     */
    public function geraRenegociacaoAjuste($renegociacao){
        try{
            if($renegociacao['VRAJUSTE']){
                $cdajuctrec  = ($renegociacao['IDAJUSTE'] == 'A')?$renegociacao['CDAJACRERENEGO']:$renegociacao['CDAJDESCRENEGO'];
                $dsrenegofin = $renegociacao['DSRENEGOFIN']?$renegociacao['DSRENEGOFIN']:'Renegociação';

                /** @var Ajctreceber $ajctreceber */
                $ajctreceber = $this->titulosAReceberFactory->insereAjctreceber(
                    $renegociacao['NRORG'], $renegociacao['CDFILIAL'], $renegociacao['NRLANCTONF'],
                    DateUtil::getDataAtual(), $cdajuctrec, $dsrenegofin, $renegociacao['VRAJUSTE'],
                    '01', '00', $renegociacao['CDCLASSFINA'], $renegociacao['CDEMPRESAEMIT']
                );

                $this->entityManager->persist($ajctreceber);
            }
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera Historico de Renegociacao
     * @param $renegociacao
     * @throws \Exception
     */
    public function geraRenegociacaoHistorico($renegociacao){
        $nrseqhistrenego = null;
        try{
            // Gera Novo Código Renegociação Financeira
            $nrseqhistrenego = $this->geraNovoCodigo($renegociacao['NRORG'], 'HISTRENEGOCIACAO'.$renegociacao['CDCLIENTE']);

            // Descrição Histórico de Renegociação Financeira
            $dsrenegofin = isset($renegociacao['DSRENEGOFIN'])?$renegociacao['DSRENEGOFIN']:null;

            // Insere Histórico de Renegociação Financeira
            /** @var Histrenegocfin $histrenegocfin */
            $histrenegocfin = $this->titulosAReceberFactory->insereHistrenegocfin(
                $renegociacao['NRORG'], $renegociacao['CDEMPRESAEMIT'], $renegociacao['CDCLIENTE'], $nrseqhistrenego,
                DateUtil::getDataAtual(), $renegociacao['CDOPERADOR'], $dsrenegofin,
                $renegociacao['CDFILIAL'], $renegociacao['NRLANCTONF']
            );

            $this->entityManager->persist($histrenegocfin);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        return $nrseqhistrenego;
    }

    /**
     * Atualiza Titulos Renegociados
     * @param $rows
     * @param $renegociacao
     * @param $titulosReneg
     * @param $titulosMarca
     * @param $nrseqhistrenego
     * @param $response
     * @throws \Exception
     */
    public function atualizaTitulosRenegociados($rows, $renegociacao, $titulosReneg, $titulosMarca, $nrseqhistrenego, $response){
        try{
            foreach($titulosMarca as $tituloMarca){
                //  Busca Contas Receber
                /** @var Contareceber $contasreceber */
                $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber(
                    $tituloMarca['NRORG'], $tituloMarca['CDFILIAL'], $tituloMarca['NRLANBOLREC']
                );

                // Gera Ajuste Boleto
                $this->ajusteBoletoService->insereAjusteReceber(
                    $tituloMarca['NRORG'], $tituloMarca['CDFILIAL'], $tituloMarca['NRLANBOLREC'], 'D',
                    $renegociacao['CDAJDESCRENEGO'], $tituloMarca['VRFINALBREC']
                );

                //  Busca Contas Receber
                /** @var Contareceber $contasreceber */
                $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                    $tituloMarca['NRORG'], $tituloMarca['CDFILIAL'], $tituloMarca['NRLANBOLREC']
                );

                /** @var Contareceber $contareceber */
                foreach ($contasreceber as $contareceber) {
                    // Atualiza Conta Receber
                    $contareceber = $this->titulosAReceberFactory->atualizaContareceberReneg(
                        $contareceber, $nrseqhistrenego, $renegociacao['CDCLIENTE']
                    );
                    $this->entityManager->persist($contareceber);
                }
            }

            // Prepara Data Source Baixa
            $baixaboleto = [
                'DTBAIXAREC'   => DateUtil::getDataAtual(),
                'DTLANCBAIXAR' => DateUtil::getDataAtual(),
                'DSBAIXAREC'   => 'Baixa automática - Renegociação Financeira',
                'IDSTATUSBREC' => 'P',
                'CDREDUCONT'   => $renegociacao['CDREDUCONT'],
                'CDEMPRESA'    => $renegociacao['CDEMPRESAEMIT'],
                'CDOPERADOR'   => $renegociacao['CDOPERADOR'],
                'CDTPBAIXAREC' => $renegociacao['CDTPBXRENEGO']
            ];

            // Baixa Titulo Planejado
            $this->baixaTitulo($renegociacao['NRORG'], $renegociacao['CDFILIAL'], $baixaboleto, $titulosMarca, $response);

            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrsaid
     * @param $nrlanctonf
     * @param $cdserie
     * @param $idstatus
     * @param $idmovfluxo
     * @param $cdclassfina
     * @param $cdtpctreceb
     * @param $dscontarec
     * @param $response
     * @throws \Exception
     */
    public function atualizaTituloAReceber(
        $nrorg, $cdfilial, $nrnotafisc, $dtemissao, $dtentrsaid, $nrlanctonf,
        $cdserie, $idstatus, $idmovfluxo, $cdclassfina,
        $cdtpctreceb, $dscontarec, $response
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Nota Fiscal
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAReceberFactory->buscaNotaFiscalReceber(
                $nrorg, $cdfilial, $nrlanctonf
            );

            // Atualiza Nota Fiscal
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAReceberFactory->atualizaNotaFiscal(
                $notafiscal, $nrnotafisc, $dtemissao, $dtentrsaid, $cdserie, $idstatus, null
            );
            $this->entityManager->persist($notafiscal);

            // Busca Contas a Receber
            /** @var Contareceber $contasreceber */
            $contasreceber = $this->titulosAReceberFactory->buscaContasreceber(
                $nrorg, $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf()
            );

            // Percorre Contas a Receber
            /** @var Contareceber $contareceber */
            foreach($contasreceber as $contareceber) {
                
                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $contareceber->getNrorg(), $contareceber->getCdempresa(),$contareceber->getCdfimovflrec(),
                    $contareceber->getCdclassfina(), $contareceber->getDtmovfluxo(), $contareceber->getVratuarec() * -1, $contareceber->getIdmovfluxo(), 'E'
                );
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();
                
                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $contareceber->getNrorg(), $contareceber->getCdempresa(),$contareceber->getCdfimovflrec(),
                    $cdclassfina, $contareceber->getDtmovfluxo(), $contareceber->getVratuarec(), $idmovfluxo, 'E'
                );
                $this->entityManager->persist($movfluxo);

                // Atualiza Conta Receber
                /** @var Contareceber $contareceber_new */
                $contareceber = $this->titulosAReceberFactory->atualizaContaReceberClassfina(
                    $contareceber, $cdtpctreceb, $contareceber->getDtatuavenrec(), $cdclassfina, $contareceber->getDtmovfluxo(), $idmovfluxo, $dscontarec
                );
                $this->entityManager->persist($contareceber);
                $this->entityManager->flush();

                /** @var Ratctreceber $ratsctreceber */
                $ratsctreceber = $this->titulosAReceberFactory->buscaRatCtReceber(
                    $nrorg, $contareceber->getNrlanctonf(), $contareceber->getCdfilial(), $contareceber->getNrseqvenrec(),
                    $contareceber->getNrparcelarec(), $notafiscal->getCdfiliemit()
                );

                // Verifica Existência de Rateio(s)
                if($ratsctreceber)
                    //Percorre Rateio(s)
                    /** @var Ratctreceber $ratctreceber */
                    foreach($ratsctreceber as $ratctreceber)
                        // Remove Rateio
                        $this->entityManager->remove($ratctreceber);
                $this->entityManager->flush();

                // Verifica Status da Nota Fiscal
                if($idstatus != 'V') {

                    // Busca Rateios da Conta a Receber
                    /** @var Ratctreceber $ratctreceber */
                    $ratctreceber = $this->geraRateioRec(
                        $nrorg, $notafiscal->getCdempresa(), $notafiscal->getCdfilial(), $notafiscal->getCdfiliemit(), $notafiscal->getNrlanctonf(),
                        $contareceber->getNrseqvenrec(), $contareceber->getNrparcelarec(), $cdtpctreceb,
                        $contareceber->getVrorigrec()
                    );

                    // Exclui Rateios da Conta a Receber
                    if($ratctreceber)
                        foreach($ratctreceber as $rateio)
                            $this->entityManager->persist($rateio);

                    // Gera Contabilização Conta Receber
                    $this->contabilizacaoService->contabilizaRecebimento(
                        $cdfilial, $contareceber->getCdempresa(), $contareceber->getNrlanctonf(), $contareceber->getNrparcelarec(),
                        $contareceber->getNrseqvenrec(), $contareceber->getIdstatrec(), $cdtpctreceb
                    );
                }else{
                    // Busca Lançamentos Contábeis
                    /** @var Lanccont $lanccont */
                    $lanccont = $this->titulosAReceberFactory->buscaLanccont(
                        $notafiscal->getNrorg(), $notafiscal->getCdempresa(), $notafiscal->getNrlanctbdoc()
                    );

                    // Exclui Lançamentos Contábeis
                    if ($lanccont)
                        foreach ($lanccont as $lancctb)
                            $this->entityManager->remove($lancctb);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Edita Titulo A Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $cdreducont
     * @param $cdtpbaixarec
     * @param $cdcartcobbol
     * @param $idcartcobrec
     * @param $idimprbolbol
     * @param $idformcadbrec
     * @param $cdins1cobbol
     * @param $cdins2cobbol
     * @param $qtdiaprotbol
     * @param $cddiaprotbol
     * @param $vrpemulbrec
     * @param $vrpemorbrec
     * @param $vrmultabrec
     * @param $vrmordiabrec
     * @throws \Exception
     */
    public function editaTituloAReceber(
        $nrorg, $cdfilial, $nrlanbolrec, $cdreducont, $cdtpbaixarec, $cdcartcobbol, $idcartcobrec,
        $idimprbolbol, $idformcadbrec, $cdins1cobbol, $cdins2cobbol, $qtdiaprotbol, $cddiaprotbol,
        $vrpemulbrec, $vrpemorbrec, $vrmultabrec, $vrmordiabrec, $dsboletrec
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Atualiza Boleto Boleto Receber
            /** @var Boletrec $boletreceber */
            $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber($nrorg, $cdfilial, $nrlanbolrec);

            $boletreceber->setCdreducont($cdreducont);
            $boletreceber->setCdtpbaixarec($cdtpbaixarec);
            $boletreceber->setCdcartcobbol($cdcartcobbol);
            $boletreceber->setIdcartcobrec($idcartcobrec);
            $boletreceber->setIdimprbolbol($idimprbolbol);
            $boletreceber->setIdformcadbrec($idformcadbrec);
            $boletreceber->setCdins1cobbol($cdins1cobbol);
            $boletreceber->setCdins2cobbol($cdins2cobbol);
            $boletreceber->setQtdiaprotbol($qtdiaprotbol);
            $boletreceber->setCddiaprotbol($cddiaprotbol);
            $boletreceber->setVrpemulbrec($vrpemulbrec);
            $boletreceber->setVrpemorbrec($vrpemorbrec);
            $boletreceber->setVrmultabrec($vrmultabrec);
            $boletreceber->setVrmordiabrec($vrmordiabrec);
            if($boletreceber->getDsboletrec() != $dsboletrec){
                $boletreceber->setDsboletrec($dsboletrec);

                // Atualiza Conta a Receber
                /** @var Contareceber $contasreceber */
                $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol($nrorg, $cdfilial, $nrlanbolrec);
                foreach($contasreceber as $contareceber){
                    $contareceber->setDscontarec($dsboletrec);
                    $this->entityManager->persist($contareceber);
                }
            }
            $this->entityManager->persist($boletreceber);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $dtvenbolrec
     * @param $boletos
     * @throws \Exception
     */
    public function validaDataVencimento($dtvenbolrec, $boletos)
    {
        try{
            $datavencimento = DateUtil::getDataDeString($dtvenbolrec);
            foreach($boletos as $boleto){

                // Busca Conta a Receber
                /** @var Contareceber $contareceber */
                $contareceber = $this->titulosAReceberFactory->buscaContareceber($boleto['NRORG'], $boleto['CDFILIAL'],$boleto['NRLANBOLREC']);

                /** @var Notafiscal $notafiscal */
                $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($contareceber->getCdfilial(),$contareceber->getNrlanctonf(),$contareceber->getNrorg());
                if (DateUtil::truncateData($datavencimento) < DateUtil::truncateData($notafiscal->getDtemissao()))
                    throw new \Exception('Data de Vencimento informada anterior a Data de Emissão do Título.');

            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera Título Associado
     * @param $nrorg
     * @param $filial
     * @param $boletos
     * @param $datavencimento
     * @param $cdreducont
     * @param $cdtpbaixarec
     * @throws \Exception
     */
    public function geraTituloAssociado($nrorg, $filial, $boletos, $datavencimento, $cdreducont, $cdtpbaixarec)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Variáveis Default
            $valortotal = 0;

            // Data de Vencimento - Objeto Date Util
            $datavencimento = DateUtil::truncateData(DateUtil::getDataDeString($datavencimento));

            // Gera Novo Código Título
            $novocodigo = $this->geraNovoCodigo($nrorg, self::PREF_NOVOCOD_BOLREC . $filial);

            // Busca Valores Títulos
            foreach ($boletos as $boleto)
                $valortotal += $boleto['VRFINALBREC'];

            /** @var $boleto BoletReceber */
            $boleto =  $this->titulosAReceberFactory->buscaBoletoReceber($nrorg, $boletos[0]['CDFILIAL'], $boletos[0]['NRLANBOLREC']);

            /** @var Boletreceber $boletonovo */
            $boletonovo = $this->titulosAReceberFactory->insereBoletoRecAssociado(
                $nrorg, $novocodigo, $filial, $valortotal, $valortotal, $datavencimento, $cdreducont, $cdtpbaixarec,
                $boleto->getCdcliente(), $boleto->getNrlicencauso(), $boleto->getCdempresa(), $boleto->getIdprotestrec(),
                self::IDBOLRECALTER_DEF, self::IDFORMCADBREC_DEF, self::STATUS_BOLETO_PEND,
                self::ID_CNAB_N_ENVIADO, self::NUM_PARCELA_INI, $boleto->getCdoperinclusao()
            );

            $this->entityManager->persist($boletonovo);
            $this->entityManager->flush();

            /** @var Boletreceber $boleto */
            foreach ($boletos as $boleto){

                // Busca Contas a Receber que compõe o Título
                /** @var Contareceber $contasreceber */
                $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC']);

                /** @var $conta ContaReceber*/
                foreach ($contasreceber as $conta){

                    if((DateUtil::comparaDatas($conta->getDtorigvenrec(),$datavencimento, 'IGUAL') == false) or
                       (DateUtil::comparaDatas($conta->getDtatuavenrec(),$datavencimento, 'IGUAL') == false)){

                        //Subtrai o valor do fluxo da conta na data anterior
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                            $conta->getDtmovfluxo(),$conta->getVrorigrec()*-1,$conta->getIdmovfluxo(), 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        //Acresce o valor do fluxo da conta na nova data de vencimento
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $conta->getCdempresa(),$conta->getCdfimovflrec(),$conta->getCdclassfina(),
                            $datavencimento,$conta->getVrorigrec(),$conta->getIdmovfluxo(), 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        /** @var Contareceber $conta */
                        $conta = $this->titulosAReceberFactory->atualizaContaRecAssoc(
                            $conta, $novocodigo, $filial, $datavencimento, $datavencimento, null
                        );

                        $this->entityManager->persist($conta);

                        /** @var Ajctreceber $ajustes */
                        $ajustes = $this->titulosAReceberFactory->buscaAjctreceber(
                            $conta->getNrorg(), $conta->getCdfilial(),$conta->getNrlanctonf(),
                            $conta->getNrseqvenrec(), $conta->getNrparcelarec()
                        );

                        if($ajustes) {
                            /** @var  $ajuste AjctReceber */
                            foreach ($ajustes as $ajuste) {

                                /** @var $tipoajuste Tipoajreceber */
                                $tipoajuste = $this->titulosAReceberFactory->buscaTipoajreceber(
                                    $ajuste->getNrorg(), $ajuste->getCdajuctrec()
                                );

                                // Verificação se o Ajuste é de Acréscimo ou Desconto
                                if ($tipoajuste->getIdajuctrec() == 'D') {
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtajusterec(), $ajuste->getVrajusterec(), 'E', 'E'
                                    );
                                } else {
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                        $ajuste->getCdclassfina(), $ajuste->getDtajusterec(), $ajuste->getVrajusterec() * -1, 'E', 'E'
                                    );
                                }

                                $this->entityManager->persist($movfluxo);
                                $this->entityManager->flush();

                                if ($tipoajuste->getIdajuctrec() == 'D') {
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                        $ajuste->getCdclassfina(), $datavencimento, $ajuste->getVrajusterec() * -1, 'E', 'E'
                                    );
                                } else {
                                    /** @var Movfluxo $movfluxo */
                                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                        $nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                        $ajuste->getCdclassfina(), $datavencimento, $ajuste->getVrajusterec(), 'E', 'E'
                                    );
                                }

                                $this->entityManager->persist($movfluxo);
                                $this->entityManager->flush();

                                $ajuste->setDtajusterec($datavencimento);

                                $this->entityManager->persist($ajuste);
                            }
                        }
                    }else{
                        $conta->setCdfilibolrec($filial);
                        $conta->setNrlanbolrec($novocodigo);

                        $this->entityManager->persist($conta);
                        $this->entityManager->flush();
                    }

                }

                /* TODO VERIFICAR ADIANTAMENTO */

                // Busca Ajustes Boleto a Receber
                /** @var Ajbolrec $ajustes */
                $ajustes = $this->titulosAReceberFactory->buscaAjbolrec(
                    $boleto['NRORG'], $boleto['CDFILIAL'],$boleto['NRLANBOLREC']
                );

                if($ajustes) {
                    /** @var Ajbolrec $ajuste */
                    foreach ($ajustes as $ajuste){

                        // Verifica se o ajuste já foi gravado no novo boleto
                        /** @var $ajusteaux Ajbolrec */
                        $ajusteaux = $this->titulosAReceberFactory->buscaAjbolrecCod(
                            $boleto['NRORG'], $filial, $novocodigo, $ajuste->getCdajuctrec()
                        );

                        if (!$ajusteaux) {
                            // Insere Ajuste Boleto a Receber
                            /** @var Ajbolrec $novoajuste */
                            $ajusteaux = $this->titulosAReceberFactory->insereAjbolrec(
                                $ajuste->getNrorg(), $filial, $novocodigo, $ajuste->getCdajuctrec(), $ajuste->getVrajbolrec()
                            );
                        } else {
                            // Atualiza Valor Ajuste Boleto a Receber
                            $ajusteaux->setVrajbolrec($ajusteaux->getVrajbolrec() + $ajuste->getVrajbolrec());
                        }
                        
                        $this->entityManager->persist($ajusteaux);
                        $this->entityManager->remove($ajuste);
                        $this->entityManager->flush();
                    }
                }

                // Apaga os boletos associados
                /** @var Boletreceber $boletreceber */
                $boletrecexcl = $this->titulosAReceberFactory->buscaBoletoReceber(
                    $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC']
                );
                if($boletrecexcl)
                    $this->entityManager->remove($boletrecexcl);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $boletos
     * @return bool|null
     * @throws \Exception
     */
    public static function verificaSitCNAB($boletos)
    {
        try{
            $exibconfirm = false;
            foreach ($boletos as $boleto)
            {
                $nrorg       = $boleto["NRORG"];
                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];

                /** @var Boletreceber $boleto */
                $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);

                if($boleto->getIdstatbrec() == 'E' and $boleto->getIdbolrecalter() == '1' )
                    throw new \Exception("Operação bloqueada! Boleto emitido não pode ser prorrogado e boleto enviado via CNAB não pode ser prorrogado sem a permissão de alteração.");

                if (((bool) $boleto->getIdsitucnabr() === true) and !(in_array($boleto->getIdsitucnabr(),array('00','01')))){

                    /** @var $geralrec GeralRec */
                    $geralrec = GeralRec::findOneByNrorg($nrorg);

                    if ($geralrec->getIdutctrlcanremr() == 'S'){
                        throw new \Exception("Operação bloqueada! Boleto enviado via CNAB não pode ser alterado, apenas prorrogado se possuir permissão de alteração.");
                    }else $exibconfirm = true;
                }

            }
            return $exibconfirm;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Cancela Associação Titulos a Receber/Recebidos
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return null
     * @throws \Exception
     */
    public function cancelaAssociarTituloARec(
        $nrorg, $cdfilial, $nrlanbolrec
    ){
        try {

            // Busca Contas a Receber
            /** @var Contareceber $contareceber */
            $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol($nrorg, $cdfilial, $nrlanbolrec);

            if(count($contasreceber)>1){

                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $nrorg);

                /** @var Ajbolrec $ajbolrec */
                $ajbolrec = $this->titulosAReceberFactory->buscaAjusteBoletoReceber($nrorg, $cdfilial, $nrlanbolrec);
                if ($ajbolrec)
                    throw new \Exception("Operação bloqueada. Não é possível dasassociar título(s) compensado(s).");

                self::separaBoletos($boletreceber);
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function cancelaRemessa(Boletreceber $boleto){
        try{
            $remessa = null;

            // Verifica Existência Remessa
            if ($boleto->getNrcnabrecrem()){

                // Atualiza Boleto Receber
                $boleto->setIdsitucnabr(self::ID_CNAB_N_ENVIADO);

                // Atualiza CNAB Receber
                /** @var $cnab Cnabrec  */
                $cnab = Cnabrec::findOneByCdempresaAndNrcnabrecAndIdtipcnabrecAndNrorg(
                    $boleto->getNrorg(), $boleto->getCdempresa(), $boleto->getNrcnabrecrem(), self::ID_TIP_CNAB_REC
                );
                $cnab->setVrtotbolcnabrec(round($cnab->getVrtotbolcnabrec() - $boleto->getVrbolrec(), 2));
                $cnab->setQtbolenvcnabrec($cnab->getQtbolenvcnabrec() -1);

                $remessa[] = [$boleto, $cnab];
            }

            return $remessa;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Separa Boletos
     * @param Boletreceber $boleto
     * @throws \Exception
     */
    public function separaBoletos(Boletreceber $boleto){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            $remessa = self::cancelaRemessa($boleto);

            if($remessa) {
                // Atualiza Boleto Receber
                $this->entityManager->persist($remessa[0]);
                // Atualiza CNAB Receber
                $this->entityManager->persist($remessa[1]);
            }

            if((int)$boleto->getNrparcbolrec() > 0)
                throw new \Exception('Operação bloqueada. Título parcelado não pode ser desassociado.');

            /** @var Boletreceber $rep */
            $rep = Boletreceber::getRepository();

            $duplicatas = $rep->getBoletoDup($boleto->getNrorg(),$boleto->getCdfilial(),$boleto->getNrlanbolrec());

            if((count($duplicatas) == 1) and($duplicatas[0]['CDFILIALDUP'] != null) and($duplicatas[0]['NRLANCTODUP'] != null))
                throw new \Exception("Contas de uma mesma duplicata não podem ser separadas.");

            $qtboletos = count($duplicatas);

            $prxboleto =  $novocodigo = NovoCodigo::geraCodigo($boleto->getNrorg(),self::PREF_NOVOCOD_BOLREC.
                $boleto->getCdfilial(),
                $qtboletos,self::TAMANHO_NRLANBOLREC);

            // Busca Contas a Receber
            /** @var Contareceber $contareceber */
            $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
            );

            $qtcontas    = 0;
            $vrajboleto  = 0;
            $vrboleto    = 0;
            $cdfilial    = 'XXX';
            $cdfilialdup = 'XXXX';
            $nrlancdup   = 'XXXXXXXXXX';
            $nrlanctonf  = 'XXXXXXXXXX';

            /** @var Contareceber $conta */
            foreach($contasreceber as $conta){
                // Grava os boletos que estÃ£o ligados a uma duplicata
                $qtcontas++;
                if( ($conta->getCdfilialdup() != null) and $conta->getNrlanctodup() ){
                    if($conta->getCdfilialdup() != $cdfilialdup or $conta->getNrlanctodup() != $nrlancdup){

                        $sqlvrboletodup = $rep->getValorBoletoDup($conta->getNrorg(),
                            $conta->getCdfildialdup(),
                            $conta->getNrlanctodup());
                        $vrboleto = $sqlvrboletodup[0]['VRATUAREC'];

                        $sqlajboldup = $rep->getAjBolDup($conta->getNrorg(),
                            $conta->getCdfildialdup(),
                            $conta->getNrlanctodup());
                        $vrajboleto = $sqlajboldup[0]['VRAJDUPBOLET'];

                        $novoboleto = new Boletreceber();
                        $novoboleto->setCdfilial($boleto->getCdfilial());
                        $novoboleto->setNrlanbolrec($prxboleto);
                        $novoboleto->setCdcliente($boleto->getCdcliente());
                        $novoboleto->setCdEmpresa($boleto->getCdempresa());
                        $novoboleto->setVrbolrec($boleto->getVrbolrec());
                        $novoboleto->setVrpemulbrec($boleto->getVrpemulbrec());
                        $novoboleto->setVrmultabrec($boleto->getVrmultabrec());
                        $novoboleto->setVrpemorbrec($boleto->getVrpemorbrec());
                        $novoboleto->setVrmordiabrec($boleto->getVrmordiabrec());
                        $novoboleto->setVrfinalbrec(round($vrboleto, 2) + round($vrajboleto, 2));
                        $novoboleto->setDtvenbolrec($conta->getDtorigvenrec());
                        $novoboleto->setCdtpbaixarec(null);
                        $novoboleto->setIdstatbrec(self::STATUS_BOLETO_PEND);
                        $novoboleto->setCdnosnumrec('');
                        $novoboleto->setIdativo('S');
                        $novoboleto->setCdoperinclusao($boleto->getCdoperinclusao());
                        $novoboleto->setDtinclusao(DateUtil::getDataAtual());
                        $novoboleto->persist();

                        $boletoatual = $prxboleto;

                        // -- Buscas os ajustes antigos para jogar nos novos boletos.
                        $sqlajboldupaux = $rep->getAjbolDupAux($conta->getCdfilialdup(),
                            $conta->getNrlanctodup(),
                            $boleto->getCdfilial(),
                            $boleto->getNrlanbolrec(),
                            \Geral\Util\Ambiente::getNrorg());

                        foreach ($sqlajboldupaux as $ajuste){
                            $novoajuste = new Ajbolrec();
                            $novoajuste->setCdfilial($boleto->getCdfilial());
                            $novoajuste->setNrlanbolrec($prxboleto);
                            $novoajuste->setCdajuctrec($ajuste['CDAJUCTREC']);
                            $novoajuste->setVrajbolrec($ajuste['VRAJDUPBOLET']);
                            $novoajuste->setDtinclusao(DateUtil::getDataAtual());
                            $this->entityManager->persist($novoajuste);

                        }
                        $prxboleto = (str_pad($prxboleto+$qtcontas,10,'0',STR_PAD_LEFT));

                        //Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $conta->getNrorg(),
                            $conta->getCdempresa(),
                            $conta->getCdfimovflrec(),
                            $conta->getCdclassfina(),
                            $conta->getDtmovfluxo(),
                            $conta->getVrorigrec() * -1,
                            $conta->getIdmovfluxo(),
                            'E'
                        );
                        $this->entityManager->persist($movfluxo);

                        //Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $conta->getNrorg(),
                            $conta->getCdempresa(),
                            $conta->getCdfimovflrec(),
                            $conta->getCdclassfina(),
                            $conta->getDtorigvenrec(),
                            $conta->getVrorigrec() * -1,
                            $conta->getIdmovfluxo(),
                            'E'
                        );
                        $this->entityManager->persist($movfluxo);

                        $conta->setNrlanbolrec($boletoatual);
                        $this->entityManager->persist($conta);
                    }
                }else{
                    if( ($cdfilial != $conta->getCdfilial()) or ($nrlanctonf != $conta->getNrlanctonf()) ){

                        //Verifica a quantidade de contas a serem atualizadas pelo procedimento  GERA_BOLETOS
                        $contaatu = $this->titulosAReceberFactory->buscaContareceberDuplicata(
                            $conta->getNrorg(),
                            $conta->getCdfilial(),
                            $conta->getCdfilibolrec(),
                            $conta->getNrlanctonf(),
                            $conta->getNrlanbolrec(),
                            null
                        );

                        $qtcontas = count($contaatu);

                        // Gera Boletos Receber
                        $this->titulosAReceberFactory->execGeraBoletos(
                            $conta->getCdfilial(), $conta->getNrlanctonf(), $conta->getCdfilial(),
                            $prxboleto, $conta->getCdtpctreceb(), 'S', $conta->getCdfilibolrec(),
                            $conta->getNrlanbolrec()
                        );

                        $cdfilial = $conta->getCdfilial();
                        $nrlanctonf = $conta->getNrlanctonf();
                        $prxboleto = (str_pad($prxboleto+$qtcontas,10,'0',STR_PAD_LEFT));
                    }

                }
            }

            /** @var Ajbolrec $delajustes */
            $delajustes =  Ajbolrec::findOneByCdfilialAndNrlanbolrecAndNrorg(
                $boleto->getCdfilial(),$boleto->getNrlanbolrec(), $boleto->getNrorg());
            if((bool)$delajustes === true)
                foreach($delajustes as $ajuste)
                    $this->entityManager->remove($ajuste);

            $this->entityManager->remove($boleto);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    public function salvarAjusteBoleto($nrorg, $cdfilial, $cdoperador, $boleto, $ajuste)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if ($boleto['NRCNABRECREM']){
                $this->excluiTituloRecRemessa($nrorg, $boleto['CDFILIAL'], $cdoperador, [$boleto]);
            }
            $this->cancelaEmissaoTituloARec($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['MANTEMNOSSONUM']);
            $nrlanbolrec = null;
            
            $this->ajusteBoletoService->rotinaInsercaoAjuste(
                $nrorg, $boleto['CDFILIAL'], $ajuste['NRLANBOLREC'],
                $ajuste['IDAJUCTREC'], $ajuste['CDAJUCTREC'], $ajuste['VRAJBOLREC']
            );
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function finalizaParcelamentoTitulos($nrorg, $cdfilial, $cdoperador, $boleto, $parcelamento)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if (isset($boleto['NRCNABRECREM'])){
                $this->excluiTituloRecRemessa($nrorg, $boleto['CDFILIAL'], $cdoperador, [$boleto]);
            }
            if(isset($boleto['IDSTATUSEMIT']))
                if($boleto['IDSTATUSEMIT'] == 'S')
                    $this->cancelaEmissaoTituloARec($nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['MANTEMNOSSONUM']);
                    
            $nrlanbolrec = null;

            foreach($parcelamento as $data){
                // Verifica Parcela Inicial
                if(($data['NRPARCBOLREC'] == '00') || ($data['NRPARCBOLREC'] == '01')) {
                    $nrorg       = $data['NRORG'];
                    $cdfilial    = $data['CDFILIAL'];
                    $nrlanbolrec = $data['NRLANBOLREC'];
                }
            }
            $this->parcelamentoBoletoService->geraParcelamentoTitulo(
                $nrorg, $boleto['CDFILIAL'], $nrlanbolrec, $parcelamento
            );
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Cancela Emissão Titulos a Receber/Recebidos
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @throws \Exception
     */
    public function cancelaEmissaoTituloARec(
        $nrorg, $cdfilial, $nrlanbolrec, $mantemnossonum = false
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Boletreceber $boletreceber */
            $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                $nrlanbolrec, $cdfilial, $nrorg
            );

            if($boletreceber->getIdstatbrec() == 'E') {
                $boletreceber->setIdstatbrec('A');
                $boletreceber->setIdsitucnabr('00');
                $boletreceber->setCdbarrarec(null);
                $boletreceber->setCddepidenrec(null);
                $boletreceber->setNrcnabrecrem(null);
                $boletreceber->setDslinhabarrec(null);
                if(!$mantemnossonum) {
                    $boletreceber->setCdnosnumrec(null);
                    $boletreceber->setIdbolemitido(null);
                }
            }

            $this->entityManager->persist($boletreceber);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }


    /**
     * Verifica Situacao CNAB Titulos A Receber/Recebidos
     * @param $boletos
     * @return bool|null
     * @throws \Exception
     */
    public function verificaCnabTituloAReceber(
        $boletos
    ){
        try {

            // Variáveis Inicialização
            $exibiconf = false;

            /** @var GeralRec $geralrec */
            $geralrec = GeralRec::findOneByNrorg($boletos[0]['NRORG']);

            // Percorre Boletos Receber
            foreach($boletos as $boleto) {

                // Busca Boleto Receber
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['NRORG']
                );

                // Verifica Situação CNAB
                if (((bool)$boletreceber->getIdsitucnabr() === true) and !(in_array($boletreceber->getIdsitucnabr(), array('00', '01')))) {
                    if ($geralrec->getIdutctrlcanremr() == 'S')
                        throw new \Exception("Operação bloqueada! Boleto enviado via CNAB não pode ser alterado, apenas prorrogado se possuir permissão de alteração.");
                    else $exibiconf = true;
                }
            }

            return $exibiconf;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Gera Novo Codigo
     * @param $nrorg
     * @param $cdcontador
     * @param int $intervalo
     * @param int $tamanho
     * @return int|null|string
     * @throws \Exception
     */
    public function geraNovoCodigo(
        $nrorg, $cdcontador, $intervalo = 1, $tamanho = 10
    ){
        try {
            // Gera Novo Código Boleto
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg, $intervalo, $tamanho);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Nota Fiscal Sistema
     * @param $cdfilial
     * @param $nrlanctonf
     * @throws \Exception
     */
    public function validaNotaSistema($nrorg, $cdfilial, $nrlanctonf)
    {
        try {
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAReceberFactory->buscaNotaFiscal(
                $nrorg, $cdfilial, $nrlanctonf
            );
            
            if ($notafiscal->getIdsistema() != 'REC')
                throw new \Exception('Operação bloqueada. Apenas títulos lançados no módulo Financeiro podem ser excluídos.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Ajuste Conta a Receber
     * @param $cdfilial
     * @param $nrlanctonf
     * @throws \Exception
     */
    public static function validaAjuste($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec)
    {
        try {
            /** @var Ajctreceber $ajctreceber */
            $ajctreceber = Ajctreceber::findOneByCdfilialAndNrlanctonfAndNrseqvenrecAndNrparcelarec($cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec);
            if ($ajctreceber)
                throw new \Exception('Operação bloqueada. Título possui ajuste(s) na Conta.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica Ajuste Boleto Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @throws \Exception
     */
    public static function verificaAjusteBoleto($nrorg, $cdfilial, $nrlanbolrec)
    {
        try {
            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = Ajbolrec::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);
            if ($ajbolrec)
                throw new \Exception('Operação bloqueada. Título possui ajuste(s).');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Boleto Receber CNAB
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @throws \Exception
     */
    public static function validaBoletoCnab($nrorg, $cdfilial, $nrlanbolrec)
    {
        try {
            /** @var Boletreceber $boletreceber */
            $boletreceber = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($nrorg, $cdfilial, $nrlanbolrec);
            if ($boletreceber->getNrcnabrecrem())
                throw new \Exception('Operação bloqueada. Título bloqueado via CNAB.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Status Conta Receber
     * @param Contareceber $contareceber
     * @throws \Exception
     */
    public static function validaStatusConta(Contareceber $contareceber)
    {
        try {
            if ($contareceber->getIdstatrec() == 'B')
                throw new \Exception('Operação bloqueada. Título baixado não pode ser excluído.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Deleta Vencimento Títulos a Receber/Recebimento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @throws \Exception
     */
    public function deletaVencTituloReceber($nrorg, $cdfilial, $nrlanbolrec, $response)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Conta a Receber
            /** @var Contareceber $contareceber */
            $contareceber = $this->titulosAReceberFactory->buscaContareceber(
                $nrorg, $cdfilial, $nrlanbolrec
            );

            // Validação Nota Fiscal
            self::validaNotaSistema($contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf());

            // Validação Conta a Receber
            self::validaStatusConta($contareceber);

            // Validação Ajuste Conta Receber
            self::validaAjuste($contareceber->getCdfilial(), $contareceber->getNrlanctonf(), $contareceber->getNrseqvenrec(), $contareceber->getNrparcelarec());

            // Validação Ajuste Boleto Receber
            self::verificaAjusteBoleto($contareceber->getNrorg(), $contareceber->getCdfilibolrec(), $contareceber->getNrlanbolrec());

            // Validação Boleto Receber CNAB
            self::validaBoletoCnab($contareceber->getNrorg(), $contareceber->getCdfilibolrec(), $contareceber->getNrlanbolrec());

            // Busca Contas a Receber
            /** @var Contareceber $contareceber */
            $contasreceber = $this->titulosAReceberFactory->buscaContasreceber(
                $contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf()
            );

            // Verifica Existência de outro(s) Vencimento(s)
            if (count($contasreceber) > 1) {

                // Busca Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg(
                    $contareceber->getCdfilial(), $contareceber->getNrlanctonf(), $contareceber->getNrorg()
                );

                // Atualiza Valor Nota Fiscal/Produto
                $notafiscal->setVrnotafisc($notafiscal->getVrnotafisc() - $contareceber->getVratuarec());
                $notafiscal->setVrprodutos($notafiscal->getVrprodutos() - $contareceber->getVratuarec());

                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $notafiscal->getCdempresa(), $notafiscal->getCdfiliemit(), $contareceber->getCdclassfina(), $contareceber->getDtatuavenrec(), $contareceber->getVratuarec() * -1, $contareceber->getIdmovfluxo(), 'E'
                );

                $this->entityManager->persist($notafiscal);

                // Busca Rateio(s) Conta a Receber
                /** @var Ratctreceber $*/
                $ratctreceber = Ratctreceber::findByCdfilialAndNrlanctonfAndNrorgAndNrseqvenrecAndNrparcelarec(
                    $contareceber->getCdfilial(), $contareceber->getNrlanctonf(), $contareceber->getNrorg(),
                    $contareceber->getNrseqvenrec(), $contareceber->getNrparcelarec()
                );

                // Deleta Rateio(s) Conta Receber
                if ($ratctreceber)
                    foreach($ratctreceber as $rateio)
                        $this->entityManager->remove($rateio);

                // Busca Boleto Receber
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $contareceber->getNrlanbolrec(), $contareceber->getCdfilibolrec(), $contareceber->getNrorg()
                );

                $this->entityManager->persist($movfluxo);

                // Deleta Conta Receber
                $this->entityManager->remove($contareceber);

                // Deleta Boleto Receber
                $this->entityManager->remove($boletreceber);

            }else throw new \Exception('Operação bloqueada. O Título possui apenas um vencimento.');


            $this->entityManager->flush();


            /** @var Contareceber $conta */
            foreach ($contasreceber as $conta) {

                // Gera Contabilização Conta Receber
                $this->contabilizacaoService->contabilizaRecebimento(
                    $conta->getCdfilial(), $conta->getCdempresa(), $conta->getNrlanctonf(), $conta->getNrparcelarec(), $conta->getNrseqvenrec(), $conta->getIdstatrec(), $conta->getCdtpctreceb()
                );

            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Deleta Documento Títulos a Receber/Recebimento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @throws \Exception
     */
    public function deletaDocTituloReceber($nrorg, $cdfilial, $nrlanbolrec)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Conta a Receber
            /** @var Contareceber $contareceber */
            $contareceber = $this->titulosAReceberFactory->buscaContareceber(
                $nrorg, $cdfilial, $nrlanbolrec
            );
            
            // Busca Contas a Receber
            /** @var Contareceber $contasreceber */
            $contasreceber = $this->titulosAReceberFactory->buscaContasreceber(
                $contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf()
            );
            
            /** @var Contareceber $conta */
            foreach ($contasreceber as $conta) {
                
                // Validação Nota Fiscal
                self::validaNotaSistema($conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf());

                // Validação Conta a Receber
                self::validaStatusConta($conta);

                // Validação Ajuste Conta Receber
                self::validaAjuste($conta->getCdfilial(), $conta->getNrlanctonf(), $conta->getNrseqvenrec(), $conta->getNrparcelarec());

                // Validação Ajuste Boleto Receber
                self::verificaAjusteBoleto($conta->getNrorg(), $conta->getCdfilibolrec(), $conta->getNrlanbolrec());

                // Validação Boleto Receber CNAB
                self::validaBoletoCnab($conta->getNrorg(), $conta->getCdfilibolrec(), $conta->getNrlanbolrec());

                /** @var Ratctreceber $ratctreceber */
                $ratctrecebers = $this->titulosAReceberFactory->buscaRatsctreceber(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                    $conta->getNrseqvenrec(), $conta->getNrparcelarec()
                );

                // Exclui Rateios Conta Receber
                if ($ratctrecebers)
                    foreach($ratctrecebers as $ratctreceber)
                        $this->entityManager->remove($ratctreceber);
                
                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtatuavenrec(), $conta->getVratuarec() * -1, $conta->getIdmovfluxo(), 'E'
                );

                $this->entityManager->persist($movfluxo);

                // Busca Conta Receber
                /** @var Contareceber $contareceber */
                $contareceber = $this->titulosAReceberFactory->buscaContareceberParcela(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                    $conta->getNrparcelarec(), $conta->getNrseqvenrec()
                );
                
                // Exclui Conta a Receber
                $this->entityManager->remove($contareceber);
                $this->entityManager->flush();

                // Busca Boleto a Receber
                /** @var Boletreceber $boletreceber */
                $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber(
                     $conta->getNrorg(), $conta->getCdfilibolrec(), $conta->getNrlanbolrec()
                );
                
                // Exclui Boleto a Receber
                $this->entityManager->remove($boletreceber);
            }
            
            /** @var Contareceber $conta */
            foreach ($contasreceber as $conta) {
                // Busca Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAReceberFactory->buscaNotafiscal(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
                );
                
                if($notafiscal){
                    // Busca Lançamentos Contábil
                    /** @var Lanccont $lanccont */
                    $lanccont = $this->titulosAReceberFactory->buscaLanccont(
                        $notafiscal->getNrorg(), $notafiscal->getCdempresa(), $notafiscal->getNrlanctbdoc()
                    );
        
                    // Exclui Lançamentos Contábil
                    if ($lanccont)
                        foreach ($lanccont as $lancctb)
                            $this->entityManager->remove($lancctb);
        
                    // Exclui Nota Fiscal Conta Receber
                    $this->entityManager->remove($notafiscal);
                    $this->entityManager->flush();
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Calcula Valor Total Baixa Recebida
     * @param $boletosmarcados
     * @return int
     */
    public static function valorTotalBaixa($boletosmarcados)
    {
        $vrmovbanc = 0;

        foreach ($boletosmarcados as $boleto)
            $vrmovbanc += $boleto['VRFINALBREC'];

        return $vrmovbanc;
    }

    /**
     * Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function baixaTituloReceber($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, $ajustesboletos = null, $contabiliza)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($ajustesboletos)
                $this->geraAjusteTitulo($ajustesboletos);
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, $contabiliza);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Gera Ajuste Título
     * @param $ajustesboletos
     * @throws \Exception
     */
    public function geraAjusteTitulo($ajustestitulo){
        try {

            // Exclui Ajustes Título
            foreach($ajustestitulo as $ajustetitulo){
                if($ajustetitulo['IDOPERACAO'] == 'E'){
                    $ajbolrec = $this->titulosAReceberFactory->buscaAjusteTitulo(
                        $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLREC'],
                        $ajustetitulo['CDAJUCTREC']
                    );
                    
                    if($ajbolrec)
                        $this->ajusteBoletoService->rotinaExclusaoAjusteReceber(
                            $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLREC'],
                            $ajustetitulo['CDAJUCTREC'], $ajustetitulo['IDAJUCTREC'], $ajustetitulo['VRAJBOLREC']
                        );
                }
            }

            // Inclui Ajustes Título
            foreach($ajustestitulo as $ajustetitulo){
                if($ajustetitulo['IDOPERACAO'] == 'I'){
                                    
                    $ajbolrec = $this->titulosAReceberFactory->buscaAjusteTitulo(
                        $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLREC'],
                        $ajustetitulo['CDAJUCTREC']
                    );
                    
                    if(!$ajbolrec)
                        $this->ajusteBoletoService->insereAjusteReceber(
                            $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLREC'],
                            $ajustetitulo['IDAJUCTREC'], $ajustetitulo['CDAJUCTREC'], $ajustetitulo['VRAJBOLREC'],
                            $ajustetitulo['IDORIGAJBREC']
                        );
                }
            }
            
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, $contabiliza = true, $transbaixa = false)
    {
        try {

            // Formata Data da Baixa Recebida e Data de Lançamento da Baixa
            $dtbaixarec   = self::formataData($baixaboleto['DTBAIXAREC']);
            $dtlancbaixar = self::formataData($baixaboleto['DTLANCBAIXAR'], false);

            // Verifica Descrição da Baixa
            $dsbaixarec = array_key_exists('DSBAIXAREC', $baixaboleto)?$baixaboleto['DSBAIXAREC']:null;
            
            // Calcula Valor Total da Baixa Recebida
            $vrbaixarec = self::valorTotalBaixa($boletosmarcados);

            // Verifica Tipo de Baixa Recebida [Real(E) / Planejada(P) / CNAB(C)]
            if (($baixaboleto['IDSTATUSBREC'] == 'E') && ($boletosmarcados[0]['IDSTATBREC'] != 'D')) {
                  // Gera Movimentação Bancária
                  $nrmovbanc = self::geraMovimentacaoBancaria($nrorg, $baixaboleto['CDREDUCONT'], $dtbaixarec, $vrbaixarec);
            }else $nrmovbanc = null;

            if(!$transbaixa){
                // Gera Novo Código Baixa
                $nrbaixarec = self::geraNovoCodigo($nrorg, 'BAIXARECEBIDA'.$cdfilial);
    
                // Atualiza Baixa Titulo(s) a Receber
                $baixarecebida = $this->titulosAReceberFactory->insereBaixaRecebida(
                    $nrorg, $cdfilial, $baixaboleto['CDEMPRESA'], $baixaboleto['CDOPERADOR'], $baixaboleto['CDTPBAIXAREC'], $baixaboleto['CDREDUCONT'], $dtlancbaixar, $nrmovbanc, $nrbaixarec,
                    $baixaboleto['IDSTATUSBREC'], $dsbaixarec, $dtbaixarec, $vrbaixarec
                );
            }else{
                $nrbaixarec = $baixaboleto['NRBAIXAREC'];
                $baixarecebida = $this->titulosAReceberFactory->buscaBaixarecebida($nrorg, $cdfilial, $nrbaixarec);
                $baixarecebida->setIdstatusbrec($baixaboleto['IDSTATUSBREC']);
                $baixarecebida->setNrmovbanc($nrmovbanc);
            }

            $this->entityManager->persist($baixarecebida);
            $this->entityManager->flush();
            
            // Verifica Tipo de Baixa [Real(E) / Planejada(P) / CNAB(C)]
            if ($baixaboleto['IDSTATUSBREC'] != 'P'){
                $idstatbrec = 'B'; // Boleto     - Baixado
                $tipofluxo  = 'R'; // Mov. Fluxo - Real
            }else{
                $idstatbrec = 'P'; // Boleto     - Planejado
                $tipofluxo  = 'E'; // Mov. Fluxo - Empenho
            }

            // Verifica Tipo de Baixa Recebida [Real(E) / Planejada(P) / CNAB(C)]
            if($baixaboleto['IDSTATUSBREC'] == 'E') {
                $notasfiscais = array();
                
                foreach ($boletosmarcados as $boleto) {
                    // Busca Título a Receber Baixado
                    /** @var Boletreceber $boletreceber */
                    $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLREC']);
                    
                    // Atualiza Título a Receber  Baixado
                    $boletreceber = $this->titulosAReceberFactory->atualizaTituloBaixado(
                        $boletreceber, $cdfilial, $idstatbrec, $dtbaixarec, $nrbaixarec,
                        $baixaboleto['CDTPBAIXAREC'], $baixaboleto['CDREDUCONT']
                    );
    
                    $this->entityManager->persist($boletreceber);

                    // Busca Contas a Receber Baixadas
                    /** @var Contareceber $contasreceber */
                    $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                        $boletreceber->getNrorg(), $boletreceber->getCdfilial(), $boletreceber->getNrlanbolrec()
                    );

                    // Percorre Contas a Receber Baixadas
                    /** @var Contareceber $conta */
                    foreach ($contasreceber as $contareceber) {
                        
                        // Atualiza Fluxo de Caixa Anterior
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfimovflrec(), $contareceber->getCdclassfina(), $contareceber->getDtmovfluxo(), $contareceber->getVrorigrec() * -1, 'E', 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Fluxo de Caixa Atual
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfimovflrec(), $contareceber->getCdclassfina(), $dtbaixarec, $contareceber->getVrorigrec(), 'R', 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Conta a Receber Baixada
                        /** @var Contareceber $contareceber */
                        $contareceber = $this->titulosAReceberFactory->atualizaContaReceber(
                            $contareceber, $idstatbrec, $tipofluxo, $dtbaixarec
                        );

                        $this->entityManager->persist($contareceber);
                        
                        /************************ TRANSMISSAO NF ************************/
                        // Busca Cliente
                        /** @var Cliente $cliente */
                        $cliente = $this->titulosAReceberFactory->buscaCliente(
                            $boletreceber->getNrorg(), $boletreceber->getCdcliente()
                        );
                        
                        // Verifica utilização da Transmissão NF na Baixa Recebida
                        if($cliente->getIdenvbolantnota() == 'S'){
                            
                            // Busca Nota Fiscal
                            /** @var Notafiscal $notafiscal */
                            $notafiscal = $this->titulosAReceberFactory->buscaNotaFiscal(
                                $contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf()
                            );
                            
                            // Busca Empresa
                            /** @var Empresa $empresa */
                            $empresa = $this->titulosAReceberFactory->buscaEmpresa(
                                $notafiscal->getNrorg(), $notafiscal->getCdempresa()
                            );
                            
                            // Constroi NF Transmissão
                            $notafiscal = array(
                                'CDFILIAL'=> $notafiscal->getCdfilial(),
                                'NRLANCTONF'=> $notafiscal->getNrlanctonf(),
                                'CDEMPRESA'=> $notafiscal->getCdempresa(),
                                'CDINSCESTA'=> $notafiscal->getCdinscesta(),
                                'CDINSCMUNI'=> $notafiscal->getCdinscmuni(),
                                'IDTPIJURNF'=> $notafiscal->getIdtpijurnf(),
                                'NRINSJURNF'=> $notafiscal->getNrinsjurnf(),
                                'CDFORNECED'=> $notafiscal->getCdforneced(),
                                'CDCLIENTE'=> $notafiscal->getCdcliente(),
                                'IDERROS'=> $notafiscal->getIderros(),
                                'IDAVISOS'=> $notafiscal->getIdavisos(),
                                'IDBLOQUEADO'=> $notafiscal->getIdbloqueado(),
                                'IDDOCFISCAL'=> $notafiscal->getIddocfiscal(),
                                'IDDOCFINANC'=> $notafiscal->getIddocfinanc(),
                                'NRNOTAFISC'=> $notafiscal->getNrnotafisc(),
                                'DTEMISSAO'=> $notafiscal->getDtemissao(),
                                'DTENTRSAID'=> $notafiscal->getDtentrsaid(),
                                'VRPRODUTOS'=> $notafiscal->getVrprodutos(),
                                'VRNOTAFISC'=> $notafiscal->getVrnotafisc(),
                                'DTINIAPUFISC'=> $notafiscal->getDtiniapufisc(),
                                'DTAPROVACAO'=> $notafiscal->getDtaprovacao(),
                                'CDOPERAPRO'=> $notafiscal->getCdoperapro(),
                                'DTCONTABIL'=> $notafiscal->getDtcontabil(),
                                'NRPLACAVEIC'=> $notafiscal->getNrplacaveic(),
                                'SGUFVEIC'=> $notafiscal->getSgufveic(),
                                'IDPGTOFRETE'=> $notafiscal->getIdpgtofrete(),
                                'DSQTDEVOLU'=> $notafiscal->getDsqtdevolu(),
                                'DSESPECVOLU'=> $notafiscal->getDsespecvolu(),
                                'DSMARCAVOLU'=> $notafiscal->getDsmarcavolu(),
                                'DSNUMEVOLU'=> $notafiscal->getDsnumevolu(),
                                'VRPBRUTOVOLU'=> $notafiscal->getVrpbrutovolu(),
                                'VRPLIQUVOLU'=> $notafiscal->getVrpliquvolu(),
                                'IDENTRSAIDOP'=> $notafiscal->getIdentrsaidop(),
                                'CDTIPOOPER'=> $notafiscal->getCdtipooper(),
                                'CDSERIE'=> $notafiscal->getCdserie(),
                                'IDTPIJURTRAN'=> $notafiscal->getIdtpijurtran(),
                                'NRINSJURTRAN'=> $notafiscal->getNrinsjurtran(),
                                'VRTTMOEDESTR'=> $notafiscal->getVrttmoedestr(),
                                'CDFISOPENTF'=> $notafiscal->getCdfisopentf(),
                                'DSCDFISOPENF'=> $notafiscal->getDscdfisopenf(),
                                'IDDOCCONTAB'=> $notafiscal->getIddoccontab(),
                                'IDSTATUS'=> $notafiscal->getIdstatus(),
                                'NRLANCTBDOC'=> $notafiscal->getNrlanctbdoc(),
                                'NRAPURACFAT'=> $notafiscal->getNrapuracfat(),
                                'IDDOCFATUR'=> $notafiscal->getIddocfatur(),
                                'IDDOCFISCENT'=> $notafiscal->getIddocfiscent(),
                                'IDDOCFISCSAI'=> $notafiscal->getIddocfiscsai(),
                                'IDDOCESTOQUE'=> $notafiscal->getIddocestoque(),
                                'CDSERIEEMIS'=> $notafiscal->getCdserieemis(),
                                'IDSISTEMA'=> $notafiscal->getIdsistema(),
                                'CDMODDOCFISC'=> $notafiscal->getCdmoddocfisc(),
                                'CDSERIEENTR'=> $notafiscal->getCdserieentr(),
                                'CDINTEGRACAO'=> $notafiscal->getCdintegracao(),
                                'CDFILIAPFATU'=> $notafiscal->getCdfiliapfatu(),
                                'NRAPFATUTNF'=> $notafiscal->getNrapfatutnf(),
                                'NRMRDPAPUR'=> $notafiscal->getNrmrdpapur(),
                                'IDORIGDESTNF'=> $notafiscal->getIdorigdestnf(),
                                'IDDOCREPLIC'=> $notafiscal->getIddocreplic(),
                                'NRLICENCAUSO'=> $notafiscal->getNrlicencauso(),
                                'CDFILIALDUP'=> $notafiscal->getCdfilialdup(),
                                'NRLANCTODUP'=> $notafiscal->getNrlanctodup(),
                                'DTSAIDEMIS'=> $notafiscal->getDtsaidemis(),
                                'NRLANCTBAJUS'=> $notafiscal->getNrlanctbajus(),
                                'NRAVISOENTR'=> $notafiscal->getNravisoentr(),
                                'DTEXPORTACAO'=> $notafiscal->getDtexportacao(),
                                'NRNOTACRED'=> $notafiscal->getNrnotacred(),
                                'CDFILIMATRIC'=> $notafiscal->getCdfilimatric(),
                                'CDTURMAPER'=> $notafiscal->getCdturmaper(),
                                'NRSEQTURPER'=> $notafiscal->getNrseqturper(),
                                'NRMATRICULA'=> $notafiscal->getNrmatricula(),
                                'CDFILESTAGIO'=> $notafiscal->getCdfilestagio(),
                                'CDESTAGIO'=> $notafiscal->getCdestagio(),
                                'CDFILIEMIT'=> $notafiscal->getCdfiliemit(),
                                'CDFILIRECE'=> $notafiscal->getCdfilirece(),
                                'NRCTRLNFCMP'=> $notafiscal->getNrctrlnfcmp(),
                                'CDAUTDESPESA'=> $notafiscal->getCdautdespesa(),
                                'CDALMOXARIFE'=> $notafiscal->getCdalmoxarife(),
                                'CDLOCALESTOQ'=> $notafiscal->getCdlocalestoq(),
                                'CDVENDEDOR'=> $notafiscal->getCdvendedor(),
                                'DTLANCAMENNF'=> $notafiscal->getDtlancamennf(),
                                'DTEMISDOCUAR'=> $notafiscal->getDtemisdocuar(),
                                'IDGEROUTDOCU'=> $notafiscal->getIdgeroutdocu(),
                                'NREXPEDICAO'=> $notafiscal->getNrexpedicao(),
                                'CDROTATRANS'=> $notafiscal->getCdrotatrans(),
                                'NRNOTADEVOL'=> $notafiscal->getNrnotadevol(),
                                'CDOPERGERA'=> $notafiscal->getCdopergera(),
                                'VRDEVOLNFUTL'=> $notafiscal->getVrdevolnfutl(),
                                'DTCOMPFATUR'=> $notafiscal->getDtcompfatur(),
                                'NMRAZSOCNF'=> $notafiscal->getNmrazsocnf(),
                                'CDOPECANCNF'=> $notafiscal->getCdopecancnf(),
                                'DTCANCELANF'=> $notafiscal->getDtcancelanf(),
                                'IDIMPONFCR'=> $notafiscal->getIdimponfcr(),
                                'NRLANCCONT'=> $notafiscal->getNrlanccont(),
                                'NRSEQLANC'=> $notafiscal->getNrseqlanc(),
                                'DSEXCCTBNF'=> $notafiscal->getDsexcctbnf(),
                                'DTEXPRNF'=> $notafiscal->getDtexprnf(),
                                'IDJIMPRNF'=> $notafiscal->getIdjimprnf(),
                                'NREXPRNF'=> $notafiscal->getNrexprnf(),
                                'CDPTOVNDFATURNF'=> $notafiscal->getCdptovndfaturnf(),
                                'DTINIREFEFAT'=> $notafiscal->getDtinirefefat(),
                                'DTFINREFEFAT'=> $notafiscal->getDtfinrefefat(),
                                'DTENTRSAIDANT'=> $notafiscal->getDtentrsaidant(),
                                'IDIMPNFPA'=> $notafiscal->getIdimpnfpa(),
                                'IDSTATUSNFE'=> $notafiscal->getIdstatusnfe(),
                                'NRNOTAFISCALELE'=> $notafiscal->getNrnotafiscalele(),
                                'NRACESSONFE'=> $notafiscal->getNracessonfe(),
                                'DSOBSSTATUSNFE'=> $notafiscal->getDsobsstatusnfe(),
                                'DSMOTIVOCANCNFE'=> $notafiscal->getDsmotivocancnfe(),
                                'CDEMPLOTENFE'=> $notafiscal->getCdemplotenfe(),
                                'NRLOTENFE'=> $notafiscal->getNrlotenfe(),
                                'IDIMPRESSODANFE'=> $notafiscal->getIdimpressodanfe(),
                                'NRPROTOCOLONFE'=> $notafiscal->getNrprotocolonfe(),
                                'IDEXPNFEP'=> $notafiscal->getIdexpnfep(),
                                'IDTIPOEMISNFE'=> $notafiscal->getIdtipoemisnfe(),
                                'IDEXPNFESANAND'=> $notafiscal->getIdexpnfesanand(),
                                'NRCPFCNPJDFS'=> $notafiscal->getNrcpfcnpjdfs(),
                                'IDEXPNFEMANAUS'=> $notafiscal->getIdexpnfemanaus(),
                                'DSNOTAFISORIGOL'=> $notafiscal->getDsnotafisorigol(),
                                'IDEMAILNFE'=> $notafiscal->getIdemailnfe(),
                                'CDOPERACTPG'=> $notafiscal->getCdoperactpg(),
                                'CDINSCESTAST'=> $notafiscal->getCdinscestast(),
                                'IDVERIFNFCONF'=> $notafiscal->getIdverifnfconf(),
                                'IDSTATUSCTBNF'=> $notafiscal->getIdstatusctbnf(),
                                'CDBARRACONTING'=> $notafiscal->getCdbarraconting(),
                                'NRSEQCAPADOC'=> $notafiscal->getNrseqcapadoc(),
                                'CDPARAMEMISRE'=> $notafiscal->getCdparamemisre(),
                                'IDNFDISPFIN'=> $notafiscal->getIdnfdispfin(),
                                'NRCPFCNPJNF'=> $notafiscal->getNrcpfcnpjnf(),
                                'CDFILIALRETRO'=> $notafiscal->getCdfilialretro(),
                                'NRAPURFATNFRETR'=> $notafiscal->getNrapurfatnfretr(),
                                'NRSEQUMAPAPUNF'=> $notafiscal->getNrsequmapapunf(),
                                'IDTPAMBNFE'=> $notafiscal->getIdtpambnfe(),
                                'VRDISPNFDEVO'=> $notafiscal->getVrdispnfdevo(),
                                'DTHRPROTOCOLNFE'=> $notafiscal->getDthrprotocolnfe(),
                                'IDEMAILNFETRANSP'=> $notafiscal->getIdemailnfetransp(),
                                'IDBLOQNFFIN'=> $notafiscal->getIdbloqnffin(),
                                'IDNFGERADAVIAXML'=> $notafiscal->getIdnfgeradaviaxml(),
                                'CDSERIERPS'=> $notafiscal->getCdserierps(),
                                'NRRPS'=> $notafiscal->getNrrps(),
                                'NRSEQLOTERPS'=> $notafiscal->getNrseqloterps(),
                                'IDDEBICREDNF'=> $notafiscal->getIddebicrednf(),
                                'DTHRREGCONTNFE'=> $notafiscal->getDthrregcontnfe(),
                                'NRREGDPECNFE'=> $notafiscal->getNrregdpecnfe(),
                                'CDOPERACTPGCAN'=> $notafiscal->getCdoperactpgcan(),
                                'DTCALCRECBRUTA'=> $notafiscal->getDtcalcrecbruta(),
                                'NRORG'=> $notafiscal->getNrorg(),
                                'DTINCLUSAO'=> $notafiscal->getDtinclusao(),
                                'DTULTATU'=> $notafiscal->getDtultatu(),
                                'NRORGINCLUSAO'=> $notafiscal->getNrorginclusao(),
                                'CDOPERINCLUSAO'=> $notafiscal->getCdoperinclusao(),
                                'NRORGULTATU'=> $notafiscal->getNrorgultatu(),
                                'CDOPERULTATU'=> $notafiscal->getCdoperultatu(),
                                'IDNFALTERADA'=> $notafiscal->getIdnfalterada(),
                                'IDNFDEVOOL'=> $notafiscal->getIdnfdevool(),
                                'IDSITUNFDEVOOL'=> $notafiscal->getIdsitunfdevool(),
                                'DSRECUSA'=> $notafiscal->getDsrecusa(),
                                'CDCHAVEACESNFPS'=> $notafiscal->getCdchaveacesnfps(),
                                'DSNFESTCANC'=> $notafiscal->getDsnfestcanc(),
                                'NRSEQUMAPAPUSN'=> $notafiscal->getNrsequmapapusn(),
                                'CDCFOPINFUSU'=> $notafiscal->getCdcfopinfusu(),
                                'DSCFOPINFUSU'=> $notafiscal->getDscfopinfusu(),
                                'IDLANCSERVICO'=> $notafiscal->getIdlancservico(),
                                'IDIMPCANCINTEGRA'=> $notafiscal->getIdimpcancintegra(),
                                'IDNFRELVENBORDO'=> $notafiscal->getIdnfrelvenbordo(),
                                'IDNFREMESSA'=> $notafiscal->getIdnfremessa(),
                                'IDNFVENDABORDO'=> $notafiscal->getIdnfvendabordo(),
                                'DSNFORIGREMESSA'=> $notafiscal->getDsnforigremessa(),
                                'IDNFCOMPRALEG'=> $notafiscal->getIdnfcompraleg(),
                                'IDEXPNFEDMSLON'=> $notafiscal->getIdexpnfedmslon(),
                                'IDNOTAFISCALOL'=> $notafiscal->getIdnotafiscalol(),
                                'CDCTRLAIDF'=> $notafiscal->getCdctrlaidf(),
                                'IDNFUTILREPLIC'=> $notafiscal->getIdnfutilreplic(),
                                'DTREFMAPAPU'=> $notafiscal->getDtrefmapapu(),
                                'DTLIBERACAOFIN'=> $notafiscal->getDtliberacaofin(),
                                'DTEXPNFSAGA'=> $notafiscal->getDtexpnfsaga(),
                                'IDEMAILXMLENVFOR'=> $notafiscal->getIdemailxmlenvfor(),
                                'IDEXPNFEISS'=> $notafiscal->getIdexpnfeiss(),
                                'CDVIATURA'=> $notafiscal->getCdviatura(),
                                'IDNFDEVOLVIDA'=> $notafiscal->getIdnfdevolvida(),
                                'IDNOTAFISIMP'=> $notafiscal->getIdnotafisimp(),
                                'DTVENCDOCUAR'=> $notafiscal->getDtvencdocuar(),
                                'NRINSJURNFPAULI'=> $notafiscal->getNrinsjurnfpauli(),
                                'CDFILILANC'=> $notafiscal->getCdfililanc(),
                                'NRLANANTECIPA'=> $notafiscal->getNrlanantecipa(),
                                'NRINSJURIEST'=> $notafiscal->getNrinsjuriest(),
                                'CDFORNECEDREME'=> $notafiscal->getCdfornecedreme(),
                                'IDTPIJURFORREME'=> $notafiscal->getIdtpijurforreme(),
                                'NRINSJURFORREME'=> $notafiscal->getNrinsjurforreme(),
                                'NRNFREMESSA'=> $notafiscal->getNrnfremessa(),
                                'IDMOTIVOCANCNF'=> $notafiscal->getIdmotivocancnf(),
                                'DSMOTIVOCANCNF'=> $notafiscal->getDsmotivocancnf(),
                                'IDPRESENCIAL'=> $notafiscal->getIdpresencial(),
                                'IDSITUCONTRATOORC'=> $notafiscal->getIdsitucontratoorc(),
                                'DSOBSSTATUSEPEC'=> $notafiscal->getDsobsstatusepec(),
                                'CDSTATUSEPEC'=> $notafiscal->getCdstatusepec(),
                                'DTHREMISSAOEPEC'=> $notafiscal->getDthremissaoepec(),
                                'SGESTADO'=> $notafiscal->getSgestado(),
                                'CDMUNICIPIO'=> $notafiscal->getCdmunicipio(),
                                'NRAPURACFATM'=> $notafiscal->getNrapuracfatm(),
                                'IDEMAILCERT'=> $notafiscal->getIdemailcert(),
                                'IDATIVO'=> $notafiscal->getIdativo(),
                                'CDOPERADORLOGADO'=> $this->environment->getCdOperador(),
                                'DSEMAILEMPRESA'=> $empresa->getDsemailempresa()
                            );
                            
                            // Agrupa NF Transmissão
                            array_push($notasfiscais, $notafiscal);
                        }
                        
                        /************************ TRANSMISSAO NF ************************/
                    }
                }

                // Atualiza Saldo da Conta Corrente
                $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                    $baixaboleto['CDREDUCONT'], $dtbaixarec, $nrorg, $vrbaixarec
                );
                $this->entityManager->flush();

                // Gera Contabilização da Baixa Recebida
                if($contabiliza === true)
                    $this->contabilizacaoService->contabilizaBaixaReceber(
                        $baixaboleto['CDEMPRESA'], $cdfilial, $cdfilial, $nrbaixarec
                    )||null;
                    
                /************************ TRANSMISSAO NF ************************/
                
                // Verifica Notas Fiscais para Transmissão
                if(count($notasfiscais) > 0){
                    
                    // Controi Transmissão NF
                    $url = str_replace('/fin/','/df/', $_SERVER['HTTP_REFERER'].'../backend/service/index.php/');
                    $data = array(
                    	"requestType" => "DataSet",
                    	"dataset" => $notasfiscais
                    );
                    
                    // Transmite NF
                    $this->callWebService($url.'transmiteBoletosNFSe', $data);
                }
                
                /************************ TRANSMISSAO NF ************************/

            } else {
                foreach ($boletosmarcados as $boleto) {
                    // Busca Título a Receber Baixado
                    /** @var Boletreceber $boletreceber */
                    $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLREC']);
                    
                    // Atualiza Título a Receber  Baixado
                    $boletreceber = $this->titulosAReceberFactory->atualizaTituloBaixado(
                        $boletreceber, $cdfilial, $idstatbrec, $dtbaixarec, $nrbaixarec,
                        $baixaboleto['CDTPBAIXAREC'], $baixaboleto['CDREDUCONT']
                    );
    
                    $this->entityManager->persist($boletreceber);

                    // Busca Contas a Receber Baixadas
                    /** @var Contareceber $contasreceber */
                    $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                        $boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLREC']
                    );

                    // Percorre Contas a Receber Baixadas
                    /** @var Contareceber $contareceber */
                    foreach ($contasreceber as $contareceber) {
                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfimovflrec(), $contareceber->getCdclassfina(), $contareceber->getDtmovfluxo(), $contareceber->getVrorigrec() * -1, 'E', 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Fluxo de Caixa
                        /** @var Movfluxo $movfluxo */
                        $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                            $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfimovflrec(), $contareceber->getCdclassfina(), $dtbaixarec, $contareceber->getVrorigrec(), 'E', 'E'
                        );

                        $this->entityManager->persist($movfluxo);
                        $this->entityManager->flush();

                        // Atualiza Conta a Receber
                        /** @var Contareceber $contareceber */
                        $contareceber = $this->titulosAReceberFactory->atualizaContaReceber(
                            $contareceber, $idstatbrec, $tipofluxo, $dtbaixarec
                        );

                        $this->entityManager->persist($contareceber);
                    }
                }
            }
            
            // Grava Imposto Baixa Recebida
            $this->titulosAReceberFactory->execGravaBaixaImposto(
                $cdfilial, $nrbaixarec, null, null, 'R'
            );

            $this->entityManager->flush();
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
    
    /**
     * Gera Movimentação Bancária
     * @param $nrorg
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $vrmovbanc
     * @return null|string
     * @throws \Exception
     */
    public function geraMovimentacaoBancaria($nrorg, $cdreducont, $dtmovbanc, $vrmovbanc)
    {
        try{
            // Novo Código Movimentação Bancária
            $nrmovbanc = self::geraNovoCodigo($nrorg, 'MOVBANCARIO'.$cdreducont);
    
            // Insere Movimentação Bancária
            /** @var Movbancario $movbancario */
            $movbancario = $this->titulosAReceberFactory->insereMovBancario(
                $nrorg, $nrmovbanc, $cdreducont, $dtmovbanc, $vrmovbanc
            );
    
            // Persist Movimentação Bancária
            $this->entityManager->persist($movbancario);
            $this->entityManager->flush();
            
            return $nrmovbanc;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Remessa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $remessatitulo
     * @param $boletosmarcados
     * @return null|string
     * @throws \Exception
     */
    public function remessaTituloReceber($nrorg, $cdfilial, $remessatitulo, $boletosmarcados)
    {
        // Gera Remessa Receber
        return $this->remessaReceberService->geraRemessaReceber(
            $nrorg, $cdfilial, $remessatitulo, $boletosmarcados
        );

    }

    /**
     * Exclusão Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBaixaTituloReceber($nrorg, $cdfilial, $boletosmarcados){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiBaixaTitulo($nrorg, $cdfilial, $boletosmarcados);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclusão Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBaixaTitulo($nrorg, $cdfilial, $boletosmarcados, $transbaixa = false){
        try {
            // Busca Parâmetros de Títulos a Receber
            /** @var GeralRec $geralrec */
            $geralrec = $this->titulosAReceberFactory->buscaGeralRec(
                $nrorg
            );
            
            // Percorre Boletos
            foreach($boletosmarcados as $boletomarcado) {

                // Busca Baixa Recebida
                /** @var Baixarecebida $baixarecebida */
                $baixarecebida = Baixarecebida::findOneByNrorgAndCdfilialAndNrbaixarec(
                    $nrorg, $boletomarcado['CDFILIBAIREC'], $boletomarcado['NRBAIXAREC']
                );

                // Verifica Existência Baixa Recebida
                if($baixarecebida) {

                    // Verifica Existência Renegociação Financeira
                    if ($baixarecebida->getNrseqhistrenego())
                        throw new \Exception('Operação bloqueada. Baixa vinculada a renegociação financeira.');

                    // Formata Data Baixa Receber
                    $dtbaixarec = $this->formataData($baixarecebida->getDtbaixarec())->format('d/m/Y');

                    if($baixarecebida->getNrlctbbrec()) {
                        // Busca Controle de Operação Contábil
                        /** @var Contopectbl $contopectbl */
                        $contopectbl = Contopectbl::getObjectsByWhere(
                            'CDEMPRESA = ? AND ? BETWEEN DTINICCTBL AND DTFINACCTBL',
                            array($baixarecebida->getCdempresa(), $dtbaixarec)
                        );

                        // Verifica Período Contábil
                        if (!$contopectbl)
                            throw new \Exception('Operação bloqueada. Data da baixa fora do período contábil financeiro.');
                    }

                    // Busca Tipo Baixa Recebida
                    /** @var Tipobaixarec $tipobaixa */
                    $tipobaixa = Tipobaixarec::findOneByNrorgAndCdtpbaixarec(
                        $baixarecebida->getNrorg(), $baixarecebida->getCdtpbaixarec()
                    );

                    if(!$transbaixa)
                        // Verifica Compensação Entre Contas
                        if ($tipobaixa->getIdbaixautilr() == 'P')
                            throw new \Exception('Operação bloqueada. Baixa compensada entre contas.');

                    // Busca Adiantamento Utilizado
                    $adiantareceb = $this->titulosAReceberFactory->validaAdiantamentoUtilizado(
                        $baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec()
                    );

                    // Valida Adiantamento Utilizado
                    if ($adiantareceb[0]['NRBAIXAREC'])
                        throw new \Exception('Operação bloqueada. Baixa possui adiantamento.');

                    // Busca Movimentação Baixa Recebida
                    /** @var $movbanc Movbancario */
                    $movbanc = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc(
                        $baixarecebida->getNrorg(), $baixarecebida->getCdreducont(), $baixarecebida->getNrmovbanc()
                    );

                    // Verifica Existência Movimentação Baixa Recebida
                    if ($movbanc)
                        if ($movbanc->getIdstconext() != 0)
                            throw new \Exception('Operação bloqueada. Movimento bancário da baixa foi conciliado.');

                    // Busca Boletos Baixa Recebida
                    /** @var $boletos Boletreceber */
                    $boletos = Boletreceber::findByNrorgAndCdfilibairecAndNrbaixarec(
                        $baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec()
                    );

                    // Percorre Boleto(s) Baixa Recebida
                    /** @var $boleto Boletreceber */
                    foreach ($boletos as $boleto) {

                        // Busca Contas a Receber
                        /** @var Contareceber $contasreceber */
                        $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                            $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
                        );

                        // Verifica Existência Conta(s) a Receber Boleto
                        if ($contasreceber) {
                            // Percorrer Conta(s) a Receber Boleto
                            /** @var Contareceber $conta */
                            foreach ($contasreceber as $conta) {

                                // Atualiza Fluxo de Caixa
                                /** @var Movfluxo $movfluxo */
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigrec() * -1, $conta->getIdmovfluxo(), 'E'
                                );

                                $this->entityManager->persist($movfluxo);
                                $this->entityManager->flush();

                                // Atualiza Fluxo de Caixa
                                /** @var Movfluxo $movfluxo */
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtatuavenrec(), $conta->getVrorigrec(), 'E', 'E'
                                );

                                $this->entityManager->persist($movfluxo);

                                // Atualiza Conta Pagar
                                $conta->setDtmovfluxo($conta->getDtatuavenrec());
                                $conta->setIdmovfluxo('E');
                                $conta->setIdstatrec('A');
                                $this->entityManager->persist($conta);
                            }
                        }

                        // Atualiza Boleto Receber
                        if ($boleto->getNrmovicred() != null)
                            $boleto->setIdstatbrec('D');
                        elseif ($boleto->getCdbarrarec() != null)
                            $boleto->setIdstatbrec('E');
                        else $boleto->setIdstatbrec('A');

                        if ($boleto->getNrcnabrecrem() == '')
                            $boleto->setIdsitucnabr('00');
                        else $boleto->setIdsitucnabr('10');

                        $boleto->setNrbaixarec(NULL);
                        $boleto->setDtliqbolrec(NULL);
                        $boleto->setCdfilibairec(NULL);
                        $this->entityManager->persist($boleto);


                        if($geralrec->getIdexclajbbxr() == 'S'){
                            // Busca Ajuste(s)
                            /** @var Ajbolrec $ajsbolrec */
                            $ajsbolrec = $this->titulosAReceberFactory->buscaAjbolrec(
                                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
                            );
                            if($ajsbolrec){
                                foreach($ajsbolrec as $ajbolrec){
                                    if($ajbolrec->getIdorigajbrec() == '2'){
                                        
                                        // Busca Tipo de Ajuste
                                        /** @var Tipoajreceber $tipoajuste */
                                        $tipoajreceber = $this->titulosAReceberFactory->buscaTipoajreceber(
                                            $ajbolrec->getNrorg(), $ajbolrec->getCdajuctrec()
                                        );
                                        
                                        $this->ajusteBoletoService->rotinaExclusaoAjuste(
                                            $ajbolrec->getNrorg(), $ajbolrec->getCdfilial(), $ajbolrec->getNrlanbolrec(),
                                            $ajbolrec->getCdajuctrec(), $tipoajreceber->getIdajuctrec(), $ajbolrec->getVrajbolrec()
                                        );
                                    }
                                }
                            }
                        }
                    }

                    // Verifica Tipo de Baixa Recebimento [Real]
                    if ($baixarecebida->getIdstatusbrec() == 'E') {

                        // Atualiza Saldo Conta Corrente
                        $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                            $baixarecebida->getCdreducont(), $baixarecebida->getDtbaixarec(), $nrorg, $baixarecebida->getVrbaixarec() * -1
                        );

                        // Busca Lançamentos Contábil
                        /** @var Lanccont $lanccont */
                        $lanccont = $this->titulosAReceberFactory->buscaLanccont(
                            $baixarecebida->getNrorg(), $baixarecebida->getCdempresa(), $baixarecebida->getNrlctbbrec()
                        );

                        // Exclui Lançamentos Contábil
                        if ($lanccont)
                            foreach ($lanccont as $lancctb)
                                $this->entityManager->remove($lancctb);
                    }

                    // Exclui Impostos
                    $impbaixarecs = $this->titulosAReceberFactory->buscaImpbaixarecBaixa($baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec());
                    if ($impbaixarecs)
                        foreach($impbaixarecs as $impbaixarec)
                            $this->entityManager->remove($impbaixarec);
                            
                    // Exclui Baixa Recebimento
                    if(!$transbaixa){
                        $this->entityManager->remove($baixarecebida);
                    }
                    $this->entityManager->flush();

                    // Verifica Existência Movimentação Baixa Recebida
                    if ($movbanc)
                        // Exclui Movimentação Baixa Recebimento
                        $this->entityManager->remove($movbanc);

                    $this->entityManager->flush();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Transforma Planejado Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function transformaPlanejadoTitulo($nrorg, $cdfilial, $boletosmarcados, $baixaboleto, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiBaixaTitulo($nrorg, $cdfilial, $boletosmarcados, true);
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, true);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBoletoBaixaTituloReceber($nrorg, $cdfilial, $boletosmarcados, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiBoletoBaixaTitulo($nrorg, $cdfilial, $boletosmarcados, $response);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function excluiBoletoBaixaTitulo($nrorg, $cdfilial, $boletosmarcados, $response, $transbaixa = false){
        try {
            // Busca Parâmetros de Títulos a Receber
            /** @var GeralRec $geralrec */
            $geralrec = $this->titulosAReceberFactory->buscaGeralRec(
                $nrorg
            );
            
            // Percorre Boletos
            foreach($boletosmarcados as $boletomarcado) {

                // Busca Baixa Recebida
                /** @var Baixarecebida $baixarecebida */
                $baixarecebida = Baixarecebida::findOneByNrorgAndCdfilialAndNrbaixarec(
                    $nrorg, $boletomarcado['CDFILIBAIREC'], $boletomarcado['NRBAIXAREC']
                );

                // Busca Boletos Baixa Recebida
                /** @var $boletos Boletreceber */
                $boletos = Boletreceber::findByNrorgAndCdfilibairecAndNrbaixarec(
                    $baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec()
                );

                // Verifica Existência de Multiplos Boletos
                if(count($boletos) > 1){

                    // Verifica Existência Baixa Recebida
                    if($baixarecebida) {

                        // Verifica Existência Renegociação Financeira
                        if ($baixarecebida->getNrseqhistrenego())
                            throw new \Exception('Operação bloqueada. Baixa vinculada a renegociação financeira.');

                        // Formata Data Baixa Receber
                        $dtbaixarec = $this->formataData($baixarecebida->getDtbaixarec())->format('d/m/Y');

                        if ($baixarecebida->getNrlctbbrec()) {
                            // Busca Controle de Operação Contábil
                            /** @var Contopectbl $contopectbl */
                            $contopectbl = Contopectbl::getObjectsByWhere(
                                'CDEMPRESA = ? AND ? BETWEEN DTINICCTBL AND DTFINACCTBL',
                                array($baixarecebida->getCdempresa(), $dtbaixarec)
                            );

                            // Verifica Período Contábil
                            if (!$contopectbl)
                                throw new \Exception('Operação bloqueada. Data da baixa fora do período contábil financeiro.');
                        }

                        // Busca Tipo Baixa Recebida
                        /** @var Tipobaixarec $tipobaixa */
                        $tipobaixa = Tipobaixarec::findOneByNrorgAndCdtpbaixarec(
                            $baixarecebida->getNrorg(), $baixarecebida->getCdtpbaixarec()
                        );

                        if(!$transbaixa)
                            // Verifica Compensação Entre Contas
                            if ($tipobaixa->getIdbaixautilr() == 'P')
                                throw new \Exception('Operação bloqueada. Baixa compensada entre contas.');

                        // Busca Adiantamento Utilizado
                        $adiantareceb = $this->titulosAReceberFactory->validaAdiantamentoUtilizado(
                            $baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec()
                        );

                        // Valida Adiantamento Utilizado
                        if ($adiantareceb[0]['NRBAIXAREC'])
                            throw new \Exception('Operação bloqueada. Baixa possui adiantamento.');

                        // Busca Movimentação Baixa Recebida
                        /** @var $movbanc Movbancario */
                        $movbanc = Movbancario::findOneByNrorgAndCdreducontAndNrmovbanc(
                            $baixarecebida->getNrorg(), $baixarecebida->getCdreducont(), $baixarecebida->getNrmovbanc()
                        );

                        // Verifica Existência Movimentação Baixa Recebida
                        if ($movbanc)
                            if ($movbanc->getIdstconext() != 0)
                                throw new \Exception('Operação bloqueada. Movimento bancário da baixa foi conciliado.');

                        // Busca Boleto Baixa Recebida
                        /** @var $boleto Boletreceber */
                        $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec(
                            $boletomarcado['NRORG'], $boletomarcado['CDFILIAL'], $boletomarcado['NRLANBOLREC']
                        );

                        // Busca Contas a Receber
                        /** @var Contareceber $contareceber */
                        $contareceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                            $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
                        );

                        // Verifica Existência Conta(s) a Receber Boleto
                        if ($contareceber) {
                            // Percorrer Conta(s) a Receber Boleto
                            /** @var Contareceber $conta */
                            foreach ($contareceber as $conta) {

                                // Atualiza Fluxo de Caixa
                                /** @var Movfluxo $movfluxo */
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtmovfluxo(), $conta->getVrorigrec() * -1, $conta->getIdmovfluxo(), 'E'
                                );

                                $this->entityManager->persist($movfluxo);
                                $this->entityManager->flush();

                                // Atualiza Fluxo de Caixa
                                /** @var Movfluxo $movfluxo */
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                                    $nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(), $conta->getDtatuavenrec(), $conta->getVrorigrec(), 'E', 'E'
                                );

                                $this->entityManager->persist($movfluxo);

                                // Atualiza Conta Pagar
                                $conta->setDtmovfluxo($conta->getDtatuavenrec());
                                $conta->setIdmovfluxo('E');
                                $conta->setIdstatrec('A');
                                $this->entityManager->persist($conta);
                            }
                        }

                        // Atualiza Boleto Receber
                        if ($boleto->getNrmovicred() != null)
                            $boleto->setIdstatbrec('D');
                        elseif ($boleto->getCdbarrarec() != null)
                            $boleto->setIdstatbrec('E');
                        else $boleto->setIdstatbrec('A');

                        if ($boleto->getNrcnabrecrem() == '')
                            $boleto->setIdsitucnabr('00');
                        else $boleto->setIdsitucnabr('10');

                        $boleto->setNrbaixarec(NULL);
                        $boleto->setDtliqbolrec(NULL);
                        $boleto->setCdfilibairec(NULL);

                        if($geralrec->getIdexclajbbxr() == 'S'){
                            // Busca Ajuste(s)
                            /** @var Ajbolrec $ajsbolrec */
                            $ajsbolrec = $this->titulosAReceberFactory->buscaAjbolrec(
                                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
                            );
                            if($ajsbolrec){
                                foreach($ajsbolrec as $ajbolrec){
                                    if($ajbolrec->getIdorigajbrec() == '2'){
                                        
                                        // Busca Tipo de Ajuste
                                        /** @var Tipoajreceber $tipoajuste */
                                        $tipoajreceber = $this->titulosAReceberFactory->buscaTipoajreceber(
                                            $ajbolrec->getNrorg(), $ajbolrec->getCdajuctrec()
                                        );
                                        
                                        $this->ajusteBoletoService->rotinaExclusaoAjuste(
                                            $ajbolrec->getNrorg(), $ajbolrec->getCdfilial(), $ajbolrec->getNrlanbolrec(),
                                            $ajbolrec->getCdajuctrec(), $tipoajreceber->getIdajuctrec(), $ajbolrec->getVrajbolrec()
                                        );
                                    }
                                }
                            }
                        }
                        
                        // Exclui Contabilizacao Baixa
                        $this->excluiContalizacaoBaixa($baixarecebida->getNrorg(), $baixarecebida->getCdempresa(), $baixarecebida->getNrlctbbrec());
                        $this->entityManager->flush();
                        
                        // Verifica Tipo de Baixa Recebimento [Real]
                        if ($baixarecebida->getIdstatusbrec() == 'E') {

                            // Atualiza Saldo Conta Corrente
                            $this->fluxoDeCaixaService->atualizaSaldoContCorr(
                                $baixarecebida->getCdreducont(), $baixarecebida->getDtbaixarec(), $nrorg, $boleto->getVrfinalbrec() * -1
                            );
                        }
                        
                        // Verifica Existência Movimentação Baixa Recebida
                        if ($movbanc){
                            // Atualiza Movimentação Baixa Recebimento
                            $movbanc->setVrmovbanc($movbanc->getVrmovbanc() - $boleto->getVrfinalbrec());
                            $this->entityManager->persist($movbanc);
                        }

                        $baixarecebida->setVrbaixarec($baixarecebida->getVrbaixarec() - $boleto->getVrfinalbrec());
                        $this->entityManager->persist($baixarecebida);

                        $this->entityManager->persist($boleto);
                        $this->entityManager->flush();

                        // Verifica Tipo de Baixa Recebimento [Real]
                        if ($baixarecebida->getIdstatusbrec() == 'E') {
                            // Gera Contabilização Baixa Recebida
                            $this->contabilizacaoService->contabilizaBaixaReceber(
                                    $baixarecebida->getCdempresa(), $baixarecebida->getCdfilial(), $cdfilial, $baixarecebida->getNrbaixarec()
                                ) || null;
                        }
                        
                        // Exclui Impostos
                        $impbaixarecs = $this->titulosAReceberFactory->buscaImpbaixarecTitulo($baixarecebida->getNrorg(), $baixarecebida->getCdfilial(), $baixarecebida->getNrbaixarec(), $boleto->getNrlanbolrec());
                        if ($impbaixarecs)
                            foreach($impbaixarecs as $impbaixarec)
                                $this->entityManager->remove($impbaixarec);
                        
                        $this->entityManager->flush();
                    }
                }else self::excluiBaixaTitulo($nrorg, $cdfilial, [$boletomarcado]);
            }
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }
    
    /**
     * Valida Adiantamento
     * @param $nrorg
     * @param $cdempresa
     * @param nrlctbbrec
     * @throws \Exception
     */
    public function excluiContalizacaoBaixa($nrorg, $cdempresa, $nrlctbbrec)
    {
        try {
            // Busca Contábil Baixa Recebida
            /** $@var $contabilizacao Lanccont */
            $lancconts = $this->titulosAReceberFactory->buscaLanccont($nrorg, $cdempresa, $nrlctbbrec);

            // Exclui Contabilização Baixa Recebida
            if ($lancconts)
                foreach($lancconts as $lanccont)
                    $this->entityManager->remove($lanccont);
                    
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Exclusão Boleto Baixa Titulo a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function transformaBoletoPlanejadoTitulo($nrorg, $cdfilial, $boletosmarcados, $baixaboleto, $response){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->excluiBoletoBaixaTitulo($nrorg, $cdfilial, $boletosmarcados, $response, true);
            $this->baixaTitulo($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response, true, true);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Título a Receber da Remessa
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @throws \Exception
     */
    public function excluiTituloRecRemessa($nrorg, $cdfilial, $cdoperador, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Percorre Todos os Boletos Marcados
            foreach ($boletos as $boleto){

                // Atualiza CNAB Boleto
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilial($boleto['NRLANBOLREC'], $boleto['CDFILIAL']);
                $boletreceber->setNrcnabrecrem(null);
                $boletreceber->setIdsitucnabr('00');
                $boletreceber->setIdbolrecalter('1');
                $this->entityManager->persist($boletreceber);

                // Atualiza CNAB
                /** @var Cnabrec $cnabrec */
                $cnabrec = Cnabrec::findOneByCdempresaAndNrcnabrec($boleto['CDEMPRESA'], $boleto['NRCNABRECREM']);
                $cnabrec->setQtbolenvcnabrec($cnabrec->getQtbolenvcnabrec() - 1);
                $cnabrec->setVrtotbolcnabrec($cnabrec->getVrtotbolcnabrec() - $boleto['VRFINALBREC']);
                $this->entityManager->persist($cnabrec);

                // Insere Log de Atualização Boleto Remessa Receber
                /** @var Logboletoremrec $logboletoremrec */
                $logboletoremrec = Logboletoremrec::findOneByNrorgAndCdfiliallogrecAndNrlanbolreclogAndCdempresalogrecAndNrcnabrecretlog(
                    $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['CDEMPRESA'], $boleto['NRCNABRECREM']
                );

                if(!$logboletoremrec) {
                    $logboletoremrec = $this->titulosAReceberFactory->insereLogBoletoRem(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['NRCNABRECREM'], $boleto['CDEMPRESA'], $cdoperador
                    );
                }else $logboletoremrec->setDtlogboletrrec(DateUtil::getDataAtual());
                $this->entityManager->persist($logboletoremrec);

            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Remessa Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @throws \Exception
     */
    public function excluiRemessaRec($nrorg, $cdfilial, $cdoperador, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Percorre Todos os Boletos Marcados
            foreach ($boletos as $boleto) {

                // Atualiza CNAB Boleto
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findByNrcnabrecremAndCdfilial($boleto['NRCNABRECREM'], $boleto['CDFILIAL']);
                if ($boletreceber) {

                    /** @var Boletreceber $boletrec */
                    foreach ($boletreceber as $boletrec) {
                        $boletrec->setNrcnabrecrem(null);
                        $boletrec->setIdsitucnabr('00');
                        $boletrec->setIdbolrecalter('1');
                        $this->entityManager->persist($boletrec);
                    }


                    // Atualiza CNAB
                    /** @var Cnabrec $cnabrec */
                    $cnabrec = Cnabrec::findOneByCdempresaAndNrcnabrec($boleto['CDEMPRESA'], $boleto['NRCNABRECREM']);
                    $cnabrec->setQtbolenvcnabrec(0);
                    $cnabrec->setVrtotbolcnabrec(0);
                    $this->entityManager->persist($cnabrec);

                    // Insere Log de Atualização Boleto Remessa Receber
                    /** @var Logboletoremrec $logboletoremrec */
                    $logboletoremrec = Logboletoremrec::findOneByNrorgAndCdfiliallogrecAndNrlanbolreclogAndCdempresalogrecAndNrcnabrecretlog(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['CDEMPRESA'], $boleto['NRCNABRECREM']
                    );

                    if (!$logboletoremrec) {
                        $logboletoremrec = $this->titulosAReceberFactory->insereLogBoletoRem(
                            $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['NRCNABRECREM'], $boleto['CDEMPRESA'], $cdoperador
                        );
                    } else $logboletoremrec->setDtlogboletrrec(DateUtil::getDataAtual());
                    $this->entityManager->persist($logboletoremrec);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Download Remessa Receber
     * @param $local_arquivo
     */
    public function downloadRemessaReceber($local_arquivo){

        $this->remessaReceberService->downloadRemessaReceber(
            $local_arquivo
        );

    }

    /**
     * Emitir Boleto
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @return null
     * @throws \Exception
     */
    public function emiteBoleto($nrorg, $cdfilial, $nrlanbolrec){

        return $this->emissaoBoletoService->emiteBoleto(
            $nrorg, $cdfilial, $nrlanbolrec
        );

    }

    /**
     * Valida Associação
     * @param $boletos
     * @throws \Exception
     */
    public function validaAssociacao($boletos)
    {
        $licensaUso    = null;
        $nrorcaevento  = null;
        $idtipodeconta = null;

        foreach ($boletos as $linha){

            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrorgAndCdfilialAndNrlanbolrec($linha['NRORG'], $linha['CDFILIAL'],$linha['NRLANBOLREC']);

            //Verifica se clientes possuem licença de uso diferentes
            if ((bool)$licensaUso === false)
                $licensaUso = $boleto->getCdCliente().$boleto->getNrlicencauso();

            if($licensaUso != ($boleto->getCdCliente().$boleto->getNrlicencauso()))
                throw new \Exception("Apenas titulos de um mesmo cliente e licença de uso podem ser associados.");

            //Verifica se há boletos com contas de tipo diferente(normal/adiantamento) marcados para associação.
            /** @var $conta Contarec */
            $conta = Contarec::findOneByCdfilibolrecAndNrlanbolrecAndNrorg(
                $linha['CDFILIAL'],$linha['NRLANBOLREC'],$linha['NRORG']
            );

            /** @var $tipodeconta Tipoctreceber */
            $tipodeconta = Tipoctreceber::findOneByCdtpctrecebAndNrorg($conta->getCdtpctreceb(),$conta->getNrorg());

            if ((bool)$idtipodeconta === false)
                $idtipodeconta = $tipodeconta->getIdtpctreceber();

            if($idtipodeconta != $tipodeconta->getIdtpctreceber())
                throw new \Exception("Operação bloqueada. Não é possível agrupar boletos de tipo de conta 'Adiantamento' com boletos de tipo de conta 'Normal'.");

            //Verifica se o boleto foi emitido
            if($boleto->getIdstatbrec() == 'E')
                throw new \Exception("Título emitido nao pode ser associado.");

            //Verifica se o boleto foi enviado via CNAB
            if (!in_array($boleto->getIdsitucnabr(),array('00','01')))
                throw new \Exception("Operação bloqueada! Título enviado via CNAB não pode ser alterado, apenas prorrogado se possuir permissão de alteração.");

            //Verifica se o boleto foi protestado
            if($boleto->getIdprotestrec() == 'S')
                throw new \Exception("Título(s) protestado(s) não pode(m) ser associado(s).");
                
            // Busca Parâmetros Gerais
            /** @var $paramgeral ParamGeral */
            $paramgeral = Paramgeral::findOneByNrorg($linha['NRORG']);
            
            // Verifica utilização Orçamento de Eventos
            if ($paramgeral->getIdexibdadorc() == 'S'){

                // Busca Nota Fiscal Relacionada ao Orçamento de Eventos
                /** @var $nfrelacorc Nfrelacorc */
                $nfrelacorc = $this->titulosAReceberFactory->buscaNfrelacorc(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
                );
                
                // Verifica existência Orçamento de Eventos
                if ($nfrelacorc){
                    
                    // Verifica Número Orçamento de Eventos
                    if (!$nrorcaevento)
                        $nrorcaevento = $nfrelacorc->getNrorcaevento();
                    
                    // Verifica Número Orçamento de Eventos Diferente
                    if ($nfrelacorc->getNrorcaevento())
                        if ($nrorcaevento != $nfrelacorc->getNrorcaevento())
                            throw new \Exception('Operação bloqueada. Títulos de orçamentos distintos não podem ser associados.');
                }
            }
        }
    }

    /**
     * Gera Adiantamento Cliente
     * @param $tituloGrid
     * @param $adiantamentoTabbed
     * @param $adiantamentoGrid
     * @throws \Exception
     */
    public function geraAdiantamentoCliente($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Valída Ajuste Adiantamento Cliente
            self::validaAjusteAdiantDevolCliente($tituloGrid, $adiantamentoTabbed);

            // Insere Ajuste Adiantamento Cliente
            self::insereAjusteAdiantDevolCliente($tituloGrid, $adiantamentoTabbed);

            $this->entityManager->flush();

            // Insere Adiantamento Cliente
            self::insereAdiantamentoCliente($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid);

            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Devolução Cliente
     * @param $tituloGrid
     * @param $devolucaoTabbed
     * @param $devolucaoGrid
     * @throws \Exception
     */
    public function geraDevolucaoCliente($tituloGrid, $devolucaoTabbed, $devolucaoGrid){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Valída Ajuste Adiantamento Cliente
            self::validaAjusteAdiantDevolCliente($tituloGrid, $devolucaoTabbed);

            // Insere Ajuste Adiantamento Cliente
            self::insereAjusteAdiantDevolCliente($tituloGrid, $devolucaoTabbed);

            $this->entityManager->flush();

            // Insere Devolução Cliente
            self::insereDevolucaoCliente($tituloGrid, $devolucaoTabbed, $devolucaoGrid);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Valída Adiantamento/Devolução Cliente
     * @param $tituloGrid
     * @param $adiantDevolTabbed
     * @throws \Exception
     */
    public function validaAjusteAdiantDevolCliente($tituloGrid, $adiantDevolTabbed){
        try {
            /** @var Ajbolrec $ajbolrec */
            $ajbolrec = $this->titulosAReceberFactory->buscaAjbolrecCod(
                $tituloGrid['NRORG'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLREC'], $adiantDevolTabbed['CDAJUCTREC']
            );

            if($ajbolrec)
                throw new \Exception('Operação bloqueada. Ajuste selecionado já cadastrado para o título, informe outro para realizar a operação.');

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Adiantamento/Devolução Cliente
     * @param $tituloGrid
     * @param $adiantDevolTabbed
     * @throws \Exception
     */
    public function insereAjusteAdiantDevolCliente($tituloGrid, $adiantDevolTabbed){
        try {
            // Insere Ajuste Ajuste Titulo
            $this->ajusteBoletoService->insereAjusteAdiantDevolCliente(
                $tituloGrid['NRORG'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLREC'], $adiantDevolTabbed['IDAJUCTREC'],
                $adiantDevolTabbed['CDAJUCTREC'], $adiantDevolTabbed['TOTALDESCONTO']
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Adiantamento Cliente
     * @param $tituloGrid
     * @param $adiantamentoTabbed
     * @param $adiantamentoGrid
     * @throws \Exception
     */
    public function insereAdiantamentoCliente($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid){
        try {
            // Percorre Adiantamento
            foreach($adiantamentoGrid as $adiantamento) {

                // Gera Novo Código - Adiantamento Receber
                $nradiantareceb = $this->geraNovoCodigo($tituloGrid['NRORG'], 'ADIANTARECEB');

                /** @var Adiantareceb $adiantareceb */
                $adiantareceb = $this->titulosAReceberFactory->insereAdiantaReceb(
                    $nradiantareceb, $adiantamento['CDFILIAL'], $tituloGrid['CDFILIAL'], $adiantamento['NRLANCTONF'],
                    $tituloGrid['NRLANBOLREC'], $adiantamento['NRSEQVENREC'], $adiantamento['NRPARCELAREC'],
                    floatval($adiantamento['VRDESCONTO']), $tituloGrid['NRORG'], $adiantamentoTabbed['CDAJUCTREC'], DateUtil::getDataAtual()
                );

                $this->entityManager->persist($adiantareceb);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Devolução Cliente
     * @param $tituloGrid
     * @param $devolucaoTabbed
     * @param $devolucaoGrid
     * @throws \Exception
     */
    public function insereDevolucaoCliente($tituloGrid, $devolucaoTabbed, $devolucaoGrid){
        try {
            // Percorre Devolução
            foreach($devolucaoGrid as $devolucao) {

                // Busca Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAReceberFactory->buscaNotafiscal(
                    $tituloGrid['NRORG'],  $devolucao['FILIDEVO'], $devolucao['NRNFDEVO']
                );

                // Valor de Devolução
                $vrdevolnfutl = $notafiscal->getVrdevolnfutl()+floatval($devolucao['VRDESCONTO']);

                // Atualiza Nota Fiscal de Devolução
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAReceberFactory->atualizaNotaFiscalDev(
                    $notafiscal, $vrdevolnfutl
                );

                $this->entityManager->persist($notafiscal);

                // Gera Novo Código - Devolução Receber
                $nrdevolucaorec =  $this->geraNovoCodigo($tituloGrid['NRORG'], 'DEVOLUCAOREC');

                /** @var Devolucaorec $devolucaorec */
                $devolucaorec = $this->titulosAReceberFactory->insereDevolucaoRec(
                    $tituloGrid['NRORG'], $devolucao['FILIDEVO'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLREC'],
                    $nrdevolucaorec, DateUtil::getDataAtual(), $devolucao['NRNFDEVO'], $devolucaoTabbed['CDAJUCTREC'],
                    floatval($devolucao['VRDESCONTO'])
                );

                $this->entityManager->persist($devolucaorec);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Prorrogação Titulo
     * @param Boletrec $boleto
     * @param $datavenc
     * @throws \Exception
     */
    private function validaProrrogacaoTitulo(Boletrec $boleto, $datavenc) {
        try {
            $menorData = DateUtil::getDataDeString('31/12/2037');

            /** @var Contareceber $contas */
            $contas = $this->titulosAReceberFactory->buscaContasreceberBol(
                $boleto->getNrorg(), $boleto->getCdfilial(), $boleto->getNrlanbolrec()
            );

            /** @var $conta Contareceber */
            foreach($contas as $conta){
                /** @var  $nota Notafiscal */
                $nota = $this->titulosAReceberFactory->buscaNotaFiscalReceber(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
                );

                if(DateUtil::comparaDatas($nota->getDtemissao(), $menorData))
                    $menorData = $nota->getDtemissao();
            }

            if(DateUtil::comparaDatas($datavenc, $menorData))
                throw new \Exception('Operação bloqueada. Data de prorrogação deve ser maior que a data de emissão da nota fiscal.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Prorroga Título a Receber
     * @param $nrorg
     * @param $boletos
     * @param $dataprorrogacao
     * @param $motivo
     * @throws \Exception
     */
    public function prorrogaTituloAReceber($nrorg, $idtppror, $dataprorrogacao, $nrdiasvenc, $motivo, $boletos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            $dataprorrogacao = $this->formataData($dataprorrogacao);

            // Percorre Títulos Marcados
            foreach ($boletos as $boleto) {
                
                // Recalcula Data - Prazo em Dias
                if($idtppror == 'P'){
                    $dtvenbolrecatual = $this->formataData($boleto['DTVENBOLREC']);
                    $dataprorrogacao = DateUtil::adicionaIntervalo($dtvenbolrecatual, $nrdiasvenc);
                }
                
                // Variaveis de Inicialização
                $cdfilial    = $boleto["CDFILIAL"];
                $nrlanbolrec = $boleto["NRLANBOLREC"];

                // Busca Titulo a Receber
                /** @var Boletreceber $boleto */
                $boleto = $this->titulosAReceberFactory->buscaBoletoReceber(
                    $nrorg, $cdfilial, $nrlanbolrec
                );

                // Valida a data de Prorrogação e caso nao gere erro updata no boleto
                $this->validaProrrogacaoTitulo($boleto, $dataprorrogacao);

                // Atualiza Vencimento Titulo
                $boleto->setDtvenbolrec($dataprorrogacao);

                // Caso o Boleto esteja com permissão de alteração altera o mesmo para alterado
                if ($boleto->getIdbolrecalter() == self::PERMITE_ALT_BOLETO)
                    $boleto->setIdbolrecalter(self::BOLETO_ALTERADO);

                $this->entityManager->persist($boleto);

                //Busca as Contas que compõem o boleto
                /** @var Contareceber $contasreceber */
                $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol(
                    $nrorg, $cdfilial, $nrlanbolrec
                );

                // Percorre Contas a Receber
                /** @var Contareceber $conta */
                foreach ($contasreceber as $conta) {

                    /** @var $conta Contareceber */
                    $dtmovfluxo = $conta->getDtmovfluxo()->format(DateUtil::FORMATO_BRASILEIRO);

                    // Retira o valor do fluxo de Caixa da Data Atual da Conta
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(),
                        $dtmovfluxo, $conta->getVrorigrec() * -1, 'E', 'E');

                    $this->entityManager->persist($movfluxo);

                    //Insere o valor da Conta no fluxo da data de ProrrogaÃ§Ã£o
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $conta->getCdempresa(), $conta->getCdfimovflrec(), $conta->getCdclassfina(),
                        $dataprorrogacao, $conta->getVrorigrec(), 'E', 'E');

                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();

                    // Seta a data de prorrogação na Conta e Executa o Commit;
                    $conta->setDtmovfluxo($dataprorrogacao);
                    $conta->setDtatuavenrec($dataprorrogacao);
                    $conta->setCdmotipror($motivo);
                    $conta->setIdmovfluxo('E');
                    $this->entityManager->persist($conta);

                    //Atualiza Fluxo dos Ajustes
                    /** @var Ajctrec $ajustes */
                    $ajustes = $this->titulosAReceberFactory->buscaAjctreceber(
                        $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),
                        $conta->getNrseqvenrec(), $conta->getNrparcelarec()
                    );
                    if ($ajustes){
                        /** @var Ajctrec $ajuste */
                        foreach ($ajustes as $ajuste) {
                            //Verificação se o Ajuste é de Acréscimo ou Desconto
                            /** @var Tipoajreceber $tipoajuste */
                            $tipoajuste = $this->titulosAReceberFactory->buscaTipoajreceber(
                                $ajuste->getNrorg(), $ajuste->getCdajuctrec()
                            );

                            //Desconto - Acrescenta o valor do ajuste na data atual e subtrai na data de ProrrogaÃ§Ã£o
                            if ($tipoajuste->getIdajuctrec() == 'D') {
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(), $ajuste->getDtajusterec(), $ajuste->getVrajusterec(), 'E', 'E');
                                $this->entityManager->persist($movfluxo);

                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(), $dataprorrogacao, $ajuste->getVrajusterec() * -1, 'E', 'E');
                                $this->entityManager->persist($movfluxo);
                            } //Acrescimo - Subtrai o valor do ajuste na data atual e acrescenta na data de ProrrogaÃ§Ã£o.
                            else {
                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(), $ajuste->getDtajusterec(), $ajuste->getVrajusterec() * -1, 'E', 'E');
                                $this->entityManager->persist($movfluxo);

                                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa($nrorg, $ajuste->getCdemmfajcntr(), $ajuste->getCdfimfajcntr(),
                                    $ajuste->getCdclassfina(), $dataprorrogacao, $ajuste->getVrajusterec(), 'E', 'E');
                                $this->entityManager->persist($movfluxo);
                            }
                            $ajuste->setDtmovfluxo($dataprorrogacao);
                            $this->entityManager->persist($ajuste);
                        }
                    }
                    $this->entityManager->flush();
                }
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $boletos
     * @param $request
     * @param $response
     * @return array
     * @throws \Exception
     */
    public function enviaTituloPorEmail($remetente, $assunto, $mensagem, $boletos, $request, $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $aux = 0;
            foreach($boletos as $boleto) {
                $caminhoBoletos = self::geraRelatorioBoleto($boleto, $request, $response, $aux);

                if (!empty($caminhoBoletos)) {
                    $this->enviaEmailTitulo($remetente, $boleto['DSEMAILCLIE'], $assunto, $mensagem, $caminhoBoletos, $boleto);
                    $this->excluiRelatorioGerados($caminhoBoletos);
                    $aux++;
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $boleto
     * @param $request
     * @param $response
     * @return array
     * @throws \Exception
     */
    public function geraRelatorioBoleto($boleto, $request, $response, $aux) {
        $caminhoBoletos = [];
        try {
            $nomeRelatorioBoleto = self::emiteBoleto($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLREC']);

            if (!empty($nomeRelatorioBoleto)) {
                $rowBirt = new Row();
                $rowBirt->set("__report_name", $nomeRelatorioBoleto);
                $rowBirt->set("CDFILIAL", $boleto['CDFILIAL']);
                $rowBirt->set("NRLANBOLREC", $boleto['NRLANBOLREC']);
                $rowBirt->set("NRORG", $boleto['NRORG']);
                $rowBirt->set("MOD_PATH", 'fin');

                $requestBirt = new Request\Row($rowBirt, "POST", "/createRemoteReport", $request->getUserId());
                $this->emissaoBoletoController->createRemoteReport($requestBirt, $response);
                
                $urlRelatorio = $response->getMessages()[$aux]->getMessage();
                
                $caminhoBoletos[] = $this->salvarRelatorio('boleto_cobranca_'.$boleto['NRORG'].$boleto['CDFILIAL'].$boleto['NRLANBOLREC'], $urlRelatorio);
            }
        } catch (\Exception $e) {
            throw $e;
        }
        return $caminhoBoletos;
    }

    /**
     * Salva Relatório
     * @param $nomeArquivo
     * @param $reportUrl
     * @return null|string
     * @throws \Exception
     */
    public function salvarRelatorio($nomeArquivo, $reportUrl) {
        $reportPath = null;
        try {

            $cURLRequest = new cURLRequest($reportUrl);
            $report = $cURLRequest->request('');
            $requestInfo = $cURLRequest->getRequestInfo();
            if ($requestInfo['http_code'] !== HTTPInterface::STATUS_CODE_OK)
                throw new \Exception('Ocorreu um erro inesperado ao salvar o relatório.');

            /*// Caminho para teste local
            $reportPath = 'C:/workspace/anexos/' . $nomeArquivo . '.pdf';*/

            if (is_dir('/tmp'))
                $reportPath = '/tmp/' . $nomeArquivo . '.pdf';
            else $reportPath = __DIR__ . DIRECTORY_SEPARATOR . $nomeArquivo . '.pdf';
            
            file_put_contents($reportPath, $report);

            unset($report);
            unset($cURLRequest);

        }catch (\Exception $e) {
            throw $e;
        }
        return $reportPath;
    }

    /**
     * Busca Mnemonico
     * @param $boleto
     * @return array
     */
    private function buscaMnemonico($boleto) {
        $endefili  = $this->titulosAReceberFactory->buscaEndefili($this->environment->getNrOrgTrab(), $this->environment->getCdFilial(), 'P');
        $municipio  = $this->titulosAReceberFactory->buscaMunicipio($endefili->getNrorg(), $endefili->getCdmunicipio())->getNmmunicipio();
        
        $replaceMnemonico = array(
            "[EMPRESA]"    => (isset($boleto['CDEMPRESA']   )) ? $this->titulosAReceberFactory->buscaEmpresa($boleto['NRORG'], $boleto['CDEMPRESA'])->getNmempresa() : '',
            "[VENCIMENTO]" => (isset($boleto['DTVENBOLREC'] )) ? self::formataData($boleto['DTVENBOLREC'])->format('d/m/Y') : '',
            "[CLIENTE]"    => (isset($boleto['CDCLIENTE']   )) ? $this->titulosAReceberFactory->buscaCliente($boleto['NRORG'], $boleto['CDCLIENTE'])->getNmrazsocclie() : '',
            "[MUNICIPIO]"  => $municipio,
            "[DIA]"        => date('d'),
            "[MES]"        => self::mesPorExtenso(date('m')),
            "[ANO]"        => date('Y'),
            "[VALOR]"      => (isset($boleto['VRFINALBREC']))  ? number_format($boleto['VRFINALBREC'],2,",",".") : ''
        );
        
        return $replaceMnemonico;
    }

    /**
     * Retorna Assunto do Email
     * @param $assunto
     * @param $boleto
     * @return string
     */
    private function retornaAssuntoEmail($assunto , $boleto) {
        $replaceMnemonico = self::buscaMnemonico($boleto);
        $assunto          = strtr($assunto, $replaceMnemonico);

        return $assunto;
    }

    /**
     * Retorna Corpo do Email
     * @param $mensagem
     * @param $boleto
     * @return string
     */
    private function retornaCorpoEmail($mensagem , $boleto) {
        $mensagem = preg_replace("/\r\n|\r|\n/",'<br/>',$mensagem);

        /** @var Empresa $empresa */
        $empresa = $this->titulosAReceberFactory->buscaEmpresa($boleto['NRORG'], $boleto['CDEMPRESA']);

        $logoempresa = $empresa->getDsdirlogomarca();
        $logoempresa = !$logoempresa?:'https://midiatest.teknisa.com/files/'.$logoempresa;

        $prefix  = '<div style="color:#000;font-weight:normal;font-style:normal;text-decoration:none;font-family:Helvetica,Arial,sans-serif;font-size:12pt;"><br><style>table{border:1px solid #cccaca;width:490px;background-color:#dbdbdb}p,pre,td{font-family:Verdana,Geneva,sans-serif !important;color:#000;font-size:14px;line-height:18px}.description{padding:20px;padding-bottom:10px;padding-top:4px}.title{padding:20px;padding-top:10px;padding-bottom:0px}.title div{font-weight:bold;width:100%;border-bottom:1px solid #000}.footer p{color:#00528C;font-weight:300;margin-bottom:40px}.header .logo{padding-bottom:10px}.logo img{width:200px;margin-left:20px;margin-top:15px}.divTable{display:table;width:100%}.divTable div{display:table-cell}</style><center><table cellpadding="0" cellspacing="0">';
        $prefix .= !$logoempresa?:'<tr class="header"><td class="logo"><p> <img dfsrc="'.$logoempresa.'" src="'.$logoempresa.'"></p></td></tr>';

        $htmlMensagem     = $prefix.'<tr> <td class="title description" style="padding-bottom: 10px;"><font face="verdana, helvetica, sans-serif" size="2">'.$mensagem.'</td> </tr> <tr class="footer"> <td> <center> <br> <p>Email automático enviado pelo SM - Service Management.</p> </center> </td> </tr> </tbody></table></center></div>';
        $replaceMnemonico = self::buscaMnemonico($boleto);
        $mensagem         = strtr($htmlMensagem, $replaceMnemonico);

        return $mensagem;
    }

    /**
     * Envia Email
     * @param $remetente
     * @param $destinatarios
     * @param $assunto
     * @param $mensagem
     * @param $anexos
     */
    public function enviaEmailTitulo($remetente, $destinatarios, $assunto, $mensagem, $anexos, $boleto) {
        $destinatarios = explode(";", trim($destinatarios));
        $this->smtp->setFrom($remetente);
        $assunto = $assunto?$assunto:[];
        foreach($anexos as $anexo)
            $this->smtp->addAttachment($anexo);
        $assunto = self::retornaAssuntoEmail($assunto, $boleto);
        $this->smtp->setSubject($assunto);
        $mensagem = self::retornaCorpoEmail($mensagem, $boleto);
        $this->smtp->sendMail($mensagem, $destinatarios);
        //var dump;
    }

    /**
     * @param $caminhoBoletos
     */
    private function excluiRelatorioGerados($caminhoBoletos){
        foreach($caminhoBoletos as $caminhoBoleto)
            if(is_file($caminhoBoleto))
                unlink($caminhoBoleto);
    }

    /**
     * Salva Titulo Descontado
     * @param $dadosDescontados
     * @param $titulos
     * @throws \Exception
     */
    public function salvaTituloDescontado($dadosDescontados, $titulos){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Variaveis de Inicialização
            $nrorg        = $dadosDescontados['NRORG'];
            $cdfilial     = $dadosDescontados['CDFILIAL'];
            $cdoperador   = $dadosDescontados['CDOPERADOR'];
            $cdtpcredbanc = $dadosDescontados['CDTPCREDBANC'];
            $dstpcredbanc = $dadosDescontados['DSTPCREDBANC'];
            $cdclassfina  = $dadosDescontados['CDCLASSFINA'];
            $dtmovicred   = $dadosDescontados['DTMOVICRED'];
            $vrmovicred   = $dadosDescontados['VRMOVICRED'];
            $cdreducont   = $dadosDescontados['CDREDUCONT'];
            $dsmovicred   = $dadosDescontados['DSMOVICRED'];
            $nrdocmbanc   = $dadosDescontados['NRDOCMBANC'];
            $cdempresa    = $dadosDescontados['CDEMPRESA'];
            $dataatual    = DateUtil::getDataAtual();

            // Valida Valor
            $valortitulos = self::validaValorCreditoXTitulos($vrmovicred, $titulos);

            // Contabiliza Crédito
            $nrlctbmcred = self::contabilizaCredito(
                $nrorg, $cdfilial, $cdempresa, $cdreducont, $dtmovicred, $vrmovicred, $cdtpcredbanc,
                $dstpcredbanc, $dsmovicred, $nrdocmbanc, $valortitulos, $cdoperador
            );

            // Gera Novo Código Movimento Bancário
            $nrmovbanc = $this->geraNovoCodigo($nrorg, 'MOVBANCARIO'.$cdreducont);

            // Insere Movimento Bancário
            /** @var Movbanc $movcredbanc */
            $movcredbanc = $this->titulosAReceberFactory->insereMovbancarioDescontado(
                $vrmovicred, $cdreducont, $nrdocmbanc, $dsmovicred, $nrorg, $dataatual, self::formataData($dtmovicred), $nrmovbanc
            );
            $this->entityManager->persist($movcredbanc);
            $this->entityManager->flush();

            // Gera Novo Código Crédito Bancário
            $nrmovicred = $this->geraNovoCodigo($nrorg, 'MOVCREDBANC'.$cdreducont);

            // Insere Credito Bancário
            /** @var Movcredbanc $movcredbanc */
            $movcredbanc = $this->titulosAReceberFactory->insereMovcredbanc(
                $cdreducont, $cdclassfina, $cdempresa, $cdfilial, $cdtpcredbanc, $dsmovicred, $vrmovicred, $nrlctbmcred,
                $cdfilial, $nrorg, $dataatual, $nrmovbanc, self::formataData($dtmovicred), $nrmovicred
            );
            $this->entityManager->persist($movcredbanc);

            // Atualiza Conta a Receber
            self::atualizaContaReceber($titulos, $nrorg, $vrmovicred, self::DESCONTADO, self::MOVFLUXO_REAL);

            // Atualiza Titulo a Receber
            self::atualizaBoletReceber($titulos, $cdreducont, $nrmovicred, $nrorg, self::CARTCOBREC, self::DESCONTADO);

            // Atualiza Saldo Conta Corrente
            $this->fluxoDeCaixaService->atualizaSaldoContCorr($cdreducont, self::formataData($dtmovicred), $nrorg, $vrmovicred);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Valida Valor do Crédito x Titulos
     * @param $vrmovicred
     * @param $titulos
     * @return int
     * @throws \Exception
     */
    public function validaValorCreditoXTitulos($vrmovicred, $titulos){
        // Variável de Inicialização
        $valortitulos = 0;

        try {
            // Soma Valores dos Títulos
            foreach ($titulos as $titulo)
                $valortitulos += $titulo['VRFINALBREC'];

            // Valida Diferença entre Valores dos Títulos x Valor do Crédito
            if (round($vrmovicred, 2) > round($valortitulos, 2))
                throw new \Exception('Operação bloqueada. Valor do Crédito deve ser menor ou igual a soma dos valores dos títulos selecionados.');

        } catch (\Exception $e) {
            throw $e;
        }

        return $valortitulos;
    }

    /**
     * Valida Valor do Crédito x Titulos
     * @param $nrorg
     * @param $cdfilial
     * @param $cdempresa
     * @param $cdreducont
     * @param $dtbaixarec
     * @param $vrmovimento
     * @param $cdtpcredbanc
     * @param $dstpcredbanc
     * @param $dsmovicred
     * @param $nrdocmbanc
     * @param $valortitulos
     * @param $cdoperador
     * @return string
     * @throws \Exception
     */
    public function contabilizaCredito(
        $nrorg, $cdfilial, $cdempresa, $cdreducont, $dtbaixarec, $vrmovimento, $cdtpcredbanc,
        $dstpcredbanc, $dsmovicred, $nrdocmbanc, $valortitulos, $cdoperador
    ){
        // Variáveis de Inicialização
        $x           = 2;
        $nrseqlanc   = 1;
        $idtipolanc  = "D";
        $contabiliza = false;
        $valor       = $vrmovimento;
        $dataatual   = DateUtil::getDataAtual();
        $ano         = DateUtil::getAno($dataatual);
        $mes         = DateUtil::getMes($dataatual);

        // Formata Data da Baixa
        $dtbaixarec = self::formataData($dtbaixarec);

        // Gera Novo Código de Lançamento Contábil
        $nrlanccont = $ano . $mes . substr($this->geraNovoCodigo($nrorg, "LANCCONTABIL" . $cdempresa . $ano . $mes), -6);

        // Busca Conta Corrent
        /** @var Contcorr $contcorr */
        $contcorr = $this->titulosAReceberFactory->buscaContcorr($nrorg, $cdreducont);
        $cdcontctbl = $contcorr->getCdcontctbl();

        // Busca Tipo de Credito Bancário
        /** @var Tpcredbanc $tpcredbanc */
        $tpcredbanc = $this->titulosAReceberFactory->buscaTpcredbanc($nrorg, $cdtpcredbanc);

        // Verifica Existência de Tipo de Crédito
        if($tpcredbanc instanceof Tpcredbanc) {
            // Verifica Parametro para Contabilização
            if ($tpcredbanc->getIdctblcrbanc() == "S") {
                $contabiliza = true;
                if($cdcontctbl == null)
                    throw new \Exception("Conta corrente não tem conta contábil cadastrada.");
            }

            // Busca Histório Padrão
            /** @var Histpadr $histpadrap */
            $histpadrap = $this->titulosAReceberFactory->buscaHistpadr($nrorg, $tpcredbanc->getCdhistpadr());

            if ($contabiliza) {
                $dslanccont = null;

                for ($i = 1; $i <= $x; $i++) {

                    // Gera Descrição do Lançamento Contábil
                    if($histpadrap instanceof Histpadr){
                        $dslanccont = $histpadrap->getDshistpadr();

                        if(strpos('[DESC]', $histpadrap->getDshistpadr()) != 0)
                            $dslanccont = str_replace('[DESC]', $dsmovicred, $dslanccont);

                        if(strpos('[TIPO]', $histpadrap->getDshistpadr()) != 0)
                            $dslanccont = str_replace('[TIPO]', $dstpcredbanc, $dslanccont);

                        if(strpos('[HIMB]', $histpadrap->getDshistpadr()) != 0)
                            $dslanccont = str_replace('[DESC]', $dsmovicred . '(' . $dstpcredbanc . ')', $dslanccont);
                    }

                    // Gera Lançamento Contabil
                    $lanccont = $this->titulosAReceberFactory->insereLanccont(
                        $nrorg, $cdempresa, $nrlanccont, str_pad($nrseqlanc, 5, 0, STR_PAD_LEFT), $cdcontctbl,
                        $tpcredbanc->getCdhistpadr(), $dtbaixarec, $nrdocmbanc, $idtipolanc, $dslanccont,
                        $valor *= -1, NULL, NULL, "N", "I", "N", $cdoperador, $dtbaixarec
                    );
                    $this->entityManager->persist($lanccont);

                    // Verifica Tipo de Lançamento Contábil
                    if($idtipolanc != "C"){
                        $idtipolanc = "C";
                        $cdcontctbl = $tpcredbanc->getCdcontctbl();
                        if($cdcontctbl == NULL)
                            throw new \Exception("Tipo de crédito bancário não tem conta contábil cadastrada.");
                    }else $idtipolanc = "D";

                    // Verifica Diferença entre Valor de Crédito x Titulos e Existência de Conta de Desconto
                    if($i == $x && $vrmovimento != $valortitulos){
                        $x          = 3;
                        $valor      = $valortitulos - $vrmovimento;
                        $cdcontctbl = $contcorr->getCdcctbcaucio();
                        if($cdcontctbl == NULL)
                            throw new \Exception("Conta corrente sem conta de desconto cadastrada.");
                    }

                    // Sequencial de Lançamento
                    $nrseqlanc++;
                }
            }
        }

        return $nrlanccont;
    }

    /**
     * Atualiza Conta a Receber
     * @param $titulos
     * @param $nrorg
     * @param $valor
     * @param $status
     * @param $idmovfluxo
     * @throws \Exception
     */
    public function atualizaContaReceber($titulos, $nrorg, $valor, $status, $idmovfluxo){
        try {
            for ($i = 0; $i < count($titulos); $i++){
                // Busca Contas a Receber
                /** @var $contasreceber Contareceber */
                $contasreceber = $this->titulosAReceberFactory->buscaContasreceberBol($nrorg, $titulos[$i]['CDFILIAL'], $titulos[$i]['NRLANBOLREC']);

                /** @var $contareceber Contareceber */
                foreach($contasreceber as $contareceber) {

                    // Atualiza Fluxo de Caixa
                    $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                        $nrorg, $contareceber->getCdempresa(), $contareceber->getCdfilial(),
                        $contareceber->getCdclassfina(), $contareceber->getDtmovfluxo(), $valor,
                        $idmovfluxo, 'E'
                    );
                    $this->entityManager->persist($movfluxo);
                    $this->entityManager->flush();

                    // Atualiza Conta a Receber
                    $contareceber->setIdstatrec($status);
                    $this->entityManager->persist($contareceber);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza Boleto a Receber
     * @param $titulos
     * @param $cdreducont
     * @param $nrmovicred
     * @param $nrorg
     * @param $cartcobrec
     * @param $status
     * @throws \Exception
     */
    public function atualizaBoletReceber($titulos, $cdreducont, $nrmovicred, $nrorg, $cartcobrec, $status){
        try {
            for ($i = 0; $i < count($titulos); $i++){
                /** @var $boletreceber Boletreceber */
                $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber($nrorg, $titulos[$i]['CDFILIAL'], $titulos[$i]['NRLANBOLREC']);
                $boletreceber->setCdreducont($cdreducont);
                $boletreceber->setNrmovicred($nrmovicred);
                $boletreceber->setIdcartcobrec($cartcobrec);
                $boletreceber->setIdstatbrec($status);

                $this->entityManager->persist($boletreceber);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Exclui Titulos Descontados
     * @param $titulosDescontados
     * @throws \Exception
     */
    public function excluiTitulosDescontados($titulosDescontados)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach($titulosDescontados as $tituloDescontado)
                self::excluiTituloDescontado($tituloDescontado);

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Titulo Descontado
     * @param $tituloDescontado
     * @throws \Exception
     */
    public function excluiTituloDescontado($tituloDescontado)
    {
        try {
            $nrorg      = $tituloDescontado['NRORG'];
            $cdreducont = $tituloDescontado['CDREDUCONT'];
            $nrmovicred = $tituloDescontado['NRMOVICRED'];

            // Busca Títulos a Receber - Conta Corrente e Movimento de Crédito
            $boletsreceber = $this->titulosAReceberFactory->buscaBoletosReceberMovCred(
                $nrorg, $cdreducont, $nrmovicred
            );

            if($boletsreceber) {
                // Valída Títulos Descontados
                self::validaTitulosDescontados($nrorg, $cdreducont, $nrmovicred);

                $boletos = array();

                /** @var $boletreceber Boletreceber */
                foreach ($boletsreceber as $boletreceber) {
                    // Agrupa Título Descontados
                    $boletos[] = array("CDFILIAL" => $boletreceber->getCdfilial(), "NRLANBOLREC" => $boletreceber->getNrlanbolrec());

                    // Verifica Status p/ Título Descontados
                    $idstatbrec = ($boletreceber->getCdbarrarec()) ? 'E' : 'A';

                    // Atualiza Títulos Descontados
                    /** @var $boletreceber_new Boletreceber */
                    $boletreceber_new = $this->titulosAReceberFactory->atualizaTituloDescontado($boletreceber, $idstatbrec);
                    $this->entityManager->persist($boletreceber_new);
                }

                // Busca Movimentação de Crédito Bancário
                /** @var $movcredbanc Movcredbanc */
                $movcredbanc = $this->titulosAReceberFactory->buscaMovcredbanc(
                    $nrorg, $cdreducont, $nrmovicred
                );

                $nrmovbanc = $movcredbanc->getNrmovbanc();
                $vrmovicred = $movcredbanc->getVrmovicred();

                // Exclui Movimentação de Crédito
                $this->entityManager->remove($movcredbanc);
                $this->entityManager->flush();

                /** @var $movbancario Movbancario */
                $movbancario = $this->titulosAReceberFactory->buscaMovbancario(
                    $nrorg, $cdreducont, $nrmovbanc
                );

                // Exclui Movimentação Bancária
                $this->entityManager->remove($movbancario);

                // Atualiza Conta a Receber
                self::atualizaContaReceber($boletos, $nrorg, $vrmovicred, self::EM_ABERTO, self::MOVFLUXO_ABERTO);

                $this->entityManager->flush();
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Título Descontado
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovicred
     * @return array
     * @throws \Exception
     */
    public function validaTitulosDescontados($nrorg, $cdreducont, $nrmovicred){
        try{
            // Variável de Inicialização
            $idstatsbrec = array();

            // Busca Titulos a Receber
            $boletsreceber = $this->titulosAReceberFactory->buscaBoletosReceberMovCred($nrorg, $cdreducont, $nrmovicred);

            /** @var Boletreceber $boletreceber */
            foreach ($boletsreceber as $boletreceber)
                $idstatsbrec[] = $boletreceber->getIdstatbrec();

            // Verifica se o Título está Baixado
            if(in_array("B", $idstatsbrec))
                throw new \Exception("Operação bloqueada. Título(s) do desconto já baixado(s).");
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $titulosDescontados
     * @throws \Exception
     */
    public function efetivarRecebimentoDescontado($titulosDescontados){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            // Variáveis de Inicialização
            $nrorg      = $titulosDescontados[0]['NRORG'];
            $cdfilial   = $titulosDescontados[0]['CDFILIAL'];
            $cdempresa  = $titulosDescontados[0]['CDEMPRESA'];
            $cdreducont = $titulosDescontados[0]['CDREDUCONT'];
            $dsbaixarec = 'Baixa temporária de título(s) descontado(s)';
            $databaixa  = DateUtil::getDataAtual();
            $vrbaixarec = 0;

            // Gera Novo Código Baixa
            $nrbaixarec = self::geraNovoCodigo($nrorg, 'BAIXARECEBIDA'.$cdfilial);

            // Insere Baixa Titulo(s) a Receber
            /** @var Baixarecebida $baixarecebida */
            $baixarecebida = $this->titulosAReceberFactory->insereBaixaRecebida(
                $nrorg, $cdfilial, $cdempresa, null, null, $cdreducont, $databaixa, null,
                $nrbaixarec, self::BAIXA_PLANEJADA, $dsbaixarec, $databaixa, $vrbaixarec
            );

            $this->entityManager->persist($baixarecebida);
            $this->entityManager->flush();

            foreach($titulosDescontados as $tituloDescontado){
                // Busca Boleto a Recebe
                /** @var Boletreceber $boletreceber */
                $boletreceber =  $this->titulosAReceberFactory->buscaBoletoReceber(
                    $tituloDescontado['NRORG'], $tituloDescontado['CDFILIAL'], $tituloDescontado['NRLANBOLREC']
                );

                // Atualiza Boleto a Receber
                $boletreceber = $this->titulosAReceberFactory->atualizaBoletoBaixa(
                    $boletreceber, $cdfilial, $nrbaixarec
                );

                $this->entityManager->flush();
                $this->entityManager->persist($boletreceber);
            }

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Baixa Titulo a Receber Descontado
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @throws \Exception
     */
    public function baixaTituloDescontadoReceber($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->baixaTituloDescontado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Cancelar Recebimento Titulo Descontado
     * @param $titulosDescontados
     * @throws \Exception
     */
    public function cancelarRecebimento($titulosDescontados){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Boletreceber $tituloDescontado */
            foreach ($titulosDescontados as $tituloDescontado) {
                // Busca Titulo a Receber
                /** @var Boletreceber $boletreceber */
                $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber(
                    $tituloDescontado["NRORG"], $tituloDescontado["CDFILIAL"], $tituloDescontado["NRLANBOLREC"]
                );

                // Atualiza Titulo a Receber
                $boletreceber->setCdfilibairec(null);
                $boletreceber->setNrbaixarec(null);

                $this->entityManager->persist($boletreceber);
            }

            $this->entityManager->flush();

            /** @var Boletreceber $tituloDescontado */
            foreach ($titulosDescontados as $tituloDescontado) {
                // Busca Baixa com Titulo a Receber
                $boletobaixa = $this->titulosAReceberFactory->buscaBoletosBaixa(
                    $tituloDescontado["NRORG"], $tituloDescontado["CDFILIBAIREC"], $tituloDescontado["NRBAIXAREC"]
                );

                // Verifica se a Baixa não possui Titulos a Receber
                if (count($boletobaixa) < 1) {
                    // Busca Baixa Recebida
                    /** @var Baixarecebida $baixarecebida */
                    $baixarecebida = $this->titulosAReceberFactory->buscaBaixarecebida(
                        $tituloDescontado["NRORG"], $tituloDescontado["NRBAIXAREC"], $tituloDescontado["CDFILIBAIREC"]
                    );

                    // Exclui Baixa Recebida
                    if ($baixarecebida)
                        $this->entityManager->remove($baixarecebida);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Cancelar Renegociação Financeira
     * @param $titulosDescontados
     * @throws \Exception
     */
    public function cancelaRenegociacaoFinanceira($titulosReneg){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            /** @var Boletreceber $tituloDescontado */
            foreach ($titulosReneg as $tituloReneg) {

                // Busca Conta do Titulo Renegociado
                /** @var Contareceber $contareceber */
                $contareceber = $this->titulosAReceberFactory->buscaContareceber(
                    $tituloReneg['NRORG'], $tituloReneg['CDFILIAL'], $tituloReneg['NRLANBOLREC']
                );

                // Busca Contas do Titulo Renegociados
                /** @var Contareceber $contasreceberrenegoc */
                $contasreceberrenegoc = $this->titulosAReceberFactory->buscaContasreceberRenegoc(
                    $contareceber->getNrorg(), $contareceber->getNrseqhistrenego(), $contareceber->getCdclienterenego()
                );

                // Busca Histórico de Renegociação
                /** @var Histrenegocfin $histrenegocfin */
                $histrenegocfin = $this->titulosAReceberFactory->buscaHistrenegocfin(
                    $contareceber->getNrorg(), $contareceber->getNrseqhistrenego(), $contareceber->getCdclienterenego()
                );

                if($histrenegocfin){
                    // Busca Contas Geradas da Renegociação
                    /** @var Contareceber $contasreceber */
                    $contasreceber = $this->titulosAReceberFactory->buscaContasreceber(
                        $histrenegocfin->getNrorg(), $histrenegocfin->getCdfilialnf(), $histrenegocfin->getNrlanctonf()
                    );

                    /** @var Contareceber $contareceber */
                    foreach($contasreceber as $contareceber){

                        // Busca Ajustes da Conta Gerada da Renegociação
                        /** @var Ajctreceber $ajsctreceber */
                        $ajsctreceber = $this->titulosAReceberFactory->buscaAjctreceber(
                            $contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getNrlanctonf(),
                            $contareceber->getNrseqvenrec(), $contareceber->getNrparcelarec()
                        );

                        /** @var Ajctreceber $ajctreceber */
                        foreach($ajsctreceber as $ajctreceber){

                            // Busca Tipo de Ajuste
                            /** @var Tipoajreceber $tipoajreceber */
                            $tipoajreceber = $this->titulosAReceberFactory->buscaTipoajreceber(
                                $ajctreceber->getNrorg(), $ajctreceber->getCdajuctrec()
                            );

                            // Exclui Ajuste da Conta a Receber
                            $this->ajusteContaService->deletaAjusteContaReceber(
                                $contareceber->getNrorg(), $contareceber->getCdfilial(), $contareceber->getCdtpctreceb(),
                                $contareceber->getNrlanbolrec(), $contareceber->getCdfilial(), $contareceber->getNrparcelarec(),
                                $contareceber->getNrseqvenrec(), $ajctreceber->getVrajusterec(), $contareceber->getNrlanctonf(),
                                $tipoajreceber->getIdajuctrec(), $ajctreceber->getVrajusterec()*-1, $ajctreceber->getVrajusterec(),
                                $contareceber->getVratuarec(), $ajctreceber->getDtajusterec(), $contareceber->getCdempresa(),
                                $ajctreceber->getCdclassfina(), $ajctreceber->getIdmovfluxo(), $ajctreceber->getCdajuctrec(),
                                $contareceber->getIdstatrec()
                            );
                        }

                        $this->entityManager->flush();

                        // Busca Titulo da Renegociação
                        /** @var Boletreceber $boletreceber */
                        $boletreceber = $this->titulosAReceberFactory->buscaBoletoReceber(
                            $contareceber->getNrorg(), $contareceber->getCdfilibolrec(), $contareceber->getNrlanbolrec()
                        );

                        if($boletreceber){
                            // Exclui Titulos Gerados da Renegociação
                            self::deletaDocTituloReceber(
                                $boletreceber->getNrorg(), $boletreceber->getCdfilial(), $boletreceber->getNrlanbolrec()
                            );
                        }
                    }

                    if($contasreceberrenegoc){
                        /** @var Contareceber $contareceberrenegoc */
                        foreach($contasreceberrenegoc as $contareceberrenegoc){
                            // Atualiza Status Título Renegociado
                            $contareceberrenegoc->setNrseqhistrenego('');
                            $contareceberrenegoc->setCdclienterenego('');
                            $this->entityManager->persist($contareceberrenegoc);
                            $this->entityManager->flush();

                            // Exclui Ajuste Titulo Renegociado
                            /** @var Ajbolrec $ajsbolrec */
                            $ajsbolrec = $this->titulosAReceberFactory->buscaAjbolrec(
                                $contareceberrenegoc->getNrorg(), $contareceberrenegoc->getCdfilibolrec(), $contareceberrenegoc->getNrlanbolrec()
                            );

                            if($ajsbolrec){
                                foreach($ajsbolrec as $ajbolrec){
                                    // Busca Tipo de Ajuste
                                    /** @var Tipoajreceber $tipoajreceber */
                                    $tipoajreceber = $this->titulosAReceberFactory->buscaTipoajreceber(
                                        $ajbolrec->getNrorg(), $ajbolrec->getCdajuctrec()
                                    );

                                    // Exclui Ajuste Titulo
                                    $this->ajusteBoletoService->rotinaExclusaoAjusteReceber(
                                        $contareceberrenegoc->getNrorg(), $contareceberrenegoc->getCdfilibolrec(), $contareceberrenegoc->getNrlanbolrec(),
                                        $ajbolrec->getCdajuctrec(), $tipoajreceber->getIdajuctrec(), $ajbolrec->getVrajbolrec()
                                    );
                                }
                            }
                        }
                    }
                    $this->entityManager->flush();
                    $this->entityManager->remove($histrenegocfin);
                }

                $this->entityManager->flush();
            }

            // Cancela Baixa Planejada Titulo Renegociação
            self::excluiBaixaTitulo($tituloReneg['NRORG'], $tituloReneg['CDFILIAL'], $titulosReneg);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial, $nrorg){
        try {
            return $this->titulosAReceberFactory->validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial, $nrorg);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Gera Titulo a Receber
     * @param $row
     * @param $response
     * @param $permiteParcela
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolrec
     * @param $parcelas
     * @return null|string
     * @throws \Exception
     */
    public function geraTituloAReceberDoc($row, $response, $permiteParcela, $nrorg, $cdfilial, $nrlanbolrec, $parcelas)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::geraTituloARec($row, $response, $permiteParcela);
            
            // Gera Parcelamento
            $this->parcelamentoBoletoService->geraParcelamentoTitulo($nrorg, $cdfilial, $nrlanbolrec, $parcelas);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Busca Todos Ajustes Boletos
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function buscaTodosAjustesBoletosRec($boletos) {
        $sql = $this->montaSqlAjuste($boletos);
        return $this->titulosAReceberFactory->buscaTodosAjustesBoletosRec($sql);
    }
    
    /**
     * Monta SQL Ajuste
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    private function montaSqlAjuste($cdFilialLanBolRec) {
        $sql = '(';
        if($cdFilialLanBolRec) {
            foreach($cdFilialLanBolRec as $value) {
                $value = $value["CDFILIAL"].$value["NRLANBOLREC"];
                $sql.= "'".$value."'".',';
            }
        }
        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    public function calcJurosSimples($nrorg, $titulo, $ajuste)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $cdfilial      = $titulo['CDFILIAL'];
            $nrlanbolrec   = $titulo['NRLANBOLREC'];
            
            
            
            
            $idajumulctrec = $ajuste['IDAJUMULCTREC'];
            $cdajumulctrec = $ajuste['CDAJUMULCTREC'];
            $vrmultabrec   = $ajuste['VRMULTABREC'];
            $idajumorctrec = $ajuste['IDAJUMORCTREC'];
            $cdajumorctrec = $ajuste['CDAJUMORCTREC'];
            $vrmordiabrec  = $ajuste['VRMORDIABREC'];
            
            if($cdajumulctrec != $cdajumorctrec){
            
                // Ajuste Multa
                /** @var Ajbolrec $ajbolrec */
                $ajbolrecmul = $this->titulosAReceberFactory->buscaAjbolrecCod($nrorg, $cdfilial, $nrlanbolrec, $cdajumulctrec);
                
                if($ajbolrecmul){
                    $vrmultabrec += $ajbolrecmul->getVrajbolrec();
                    $this->ajusteBoletoService->rotinaExclusaoAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajumulctrec, $idajumulctrec, $vrmultabrec);
                }
                
                $this->ajusteBoletoService->rotinaInsercaoAjuste(
                    $nrorg, $cdfilial, $nrlanbolrec, $idajumulctrec, $cdajumulctrec, $vrmultabrec
                );
                
                // Ajuste Mora
                /** @var Ajbolrec $ajbolrec */
                $ajbolrecmor = $this->titulosAReceberFactory->buscaAjbolrecCod($nrorg, $cdfilial, $nrlanbolrec, $cdajumorctrec);
                
                if($ajbolrecmor){
                    $vrmordiabrec += $ajbolrecmor->getVrajbolrec();
                    $this->ajusteBoletoService->rotinaExclusaoAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajumorctrec, $idajumorctrec, $vrmordiabrec);
                }
                
                $this->ajusteBoletoService->rotinaInsercaoAjuste(
                    $nrorg, $cdfilial, $nrlanbolrec, $idajumorctrec, $cdajumorctrec, $vrmordiabrec
                );
            }else{
                $vrmordiabrec = $vrmultabrec + $vrmordiabrec;
                
                /** @var Ajbolrec $ajbolrec */
                $ajbolrecmor = $this->titulosAReceberFactory->buscaAjbolrecCod($nrorg, $cdfilial, $nrlanbolrec, $cdajumorctrec);
                
                if($ajbolrecmor){
                    $vrmordiabrec += $ajbolrecmor->getVrajbolrec();
                    $this->ajusteBoletoService->rotinaExclusaoAjusteReceber($nrorg, $cdfilial, $nrlanbolrec, $cdajumorctrec, $idajumorctrec, $vrmordiabrec);
                }
                
                $this->ajusteBoletoService->rotinaInsercaoAjuste(
                    $nrorg, $cdfilial, $nrlanbolrec, $idajumorctrec, $cdajumorctrec, $vrmordiabrec
                );
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function calcJurosComposto($nrorg, $titulo, $ajuste)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $cdfilial     = $titulo['CDFILIAL'];
            $nrlanbolrec  = $titulo['NRLANBOLREC'];
            $dtvencimento = $titulo['DTVENBOLPAG'];
            $dtprevisao   = $titulo['DTVENBOLPAG'];
            $vrfinalbrec  = $titulo['VRFINALBREC'];
            
            if($ajuste['IDJUROSCOMP'] == 'P'){
                $vrajbolrec = $this->calcJurosCompPeFixo(
                    $dtvencimento, $dtprevisao, $ajuste['VRPERCENTUAL'], $vrfinalbrec
                );
            }else{
                $vrajbolrec = $this->calcJurosCompIndice(
                    $dtvencimento, $dtprevisao, $ajuste['CDINDICE'], $vrfinalbrec
                );
            }
            
            // Ajuste
            $idajuctrec = $ajuste['IDAJUCTREC'];
            $cdajuctrec = $ajuste['CDAJUCTREC'];
            
            $this->ajusteBoletoService->rotinaInsercaoAjuste(
                $nrorg, $cdfilial, $nrlanbolrec, $idajuctrec, $cdajuctrec, $vrajbolrec
            );
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function calcJurosCompPeFixo($dtvencimento, $dtprevisao, $vrperc, $vrorig) {
        $result = 0;
       
        if($vrperc > 0){
            $qtdia   = $dtprevisao - $dtvencimento;
            $qtmes   = $qtdia / 30;
            $restdia = $qtdia - ($qtmes * 30);
            $vrmes   = $vrorig;
            $vrdia   = 0;
        
            for($count = 1; $count < $qtmes; $count++)
                $vrmes = $vrmes + ($vrmes * ($vrperc/100));
            
            if ($restdia > 0)
                $vrdia = $vrmes * $restdia * (($vrperc/100) / 30);
            
            $result = $vrmes + $vrdia;
        }
        
        return $result;
    }
    
    public function calcJurosCompIndice($dtvencimento, $dtprevisao, $cdindice, $vrorig) {
        $result = 0;
       
        $qtdia   = $dtprevisao - $dtvencimento;
        $qtmes   = $qtdia / 30;
        $restdia = $qtdia - ($qtmes * 30);
        $vrmes   = $vrorig;
        $vrdia   = 0;
        
        $movindice = $this->titulosAReceberFactory->buscaMovindice(
            $cdindice, $dtvencbol, $dtprevisao
        );
        
        if($movindice){
            if (count($movindice) == 1){
                for($count = 1; $count < $qtmes; $count++){
                    if ($movindice[$count]['VRINDICE'])
                          $vrmes = $vrmes + ($vrmes * ($movindice[$count]['VRINDICE'] / 100));
                }          
            }else{
                for($count = 1; $count < $qtmes; $count++){
                   $dtvencbol = $dtvencbol + 30;
                   
                   //to do
                   
                   if (['VRINDICE'])
                      $vrmes = $vrmes + ($vrmes * ($movindice[$count]['VRINDICE']/100));
                
                }
            }
    
             if ($restdia > 0)
                if ($movindice[$count]['VRINDICE'])
                   $vrdia = $vrmes * $restdia * (($movindice[count($movindice)]['VRINDICE']/100) / 30);
    
             $result = $vrmes + $vrdia;
        }
        
       return $result;
    
    }
    
    /* Cadastrar Guia de Tributo
     * @param $dadosTributo
     * @param $titulosGuiaTributo
     * @throws \Exception
     */
    public function salvaAnexoTitulo($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row['NRANEXOLANCBOL'] = $this->geraNovoCodigo($row['NRORG'], 'ANEXOLANCBOL');
            $row['CDTELA'] = 'REC00001';
            //Salva o arquivo sempre como txt independente do seu formato
            $row['NMPATHANEXO'] = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row['CDTELA'] . $row['NRANEXOLANCBOL'] .'.txt';
            $row['DSINFOANEXO'] = $row['NMANEXOLANCBOL'][0]["lastModified"] . '_' . $row['NMANEXOLANCBOL'][0]["lastModifiedDate"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["webkitRelativePath"]. '_' . $row['NMANEXOLANCBOL'][0]["type"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["size"] . '_' . $row['NMANEXOLANCBOL'][0]["name"];
            
            $this->titulosAReceberFactory->salvaAnexoTitulo($row);
            
            file_put_contents($row['NMPATHANEXO'], $row['NMANEXOLANCBOL'][0]["b64File"]);
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function salvaAnexoGrid($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row['NRANEXOLANCBOL'] = $this->geraNovoCodigo($row['NRORG'], 'ANEXOLANCBOL');
            //Salva o arquivo sempre como txt independente do seu formato
            $row['DSINFOANEXO'] = $row['NMANEXOLANCBOL'][0]["lastModified"] . '_' . $row['NMANEXOLANCBOL'][0]["lastModifiedDate"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["webkitRelativePath"]. '_' . $row['NMANEXOLANCBOL'][0]["type"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["size"] . '_' . $row['NMANEXOLANCBOL'][0]["name"];
            
            
            $row['NMPATHANEXO'] = $this->saveXmlInFileUplouder($row['NMANEXOLANCBOL'][0]);
            
            $this->titulosAReceberFactory->salvaAnexoGrid($row);
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deletaAnexoTitulo($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                if(file_exists($row['NMPATHANEXO'])){
                    unlink($row['NMPATHANEXO']);
                }
                
                $this->titulosAReceberFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteAnexoGrid($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                
                $this->fileUploader->deleteFile($row['NMPATHANEXO']);
                
                $this->titulosAReceberFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deletaAnexoInFileServer($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                
                $this->deletaAnexoTitulo($row);
                
                $this->titulosAReceberFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteAnexoIncompletos(){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $anexosIncompletos = $this->titulosAReceberFactory->selectDeleteAnexoIncompletos();
            
            foreach ($anexosIncompletos as $anexo){
                if(file_exists($anexo['NMPATHANEXO'])){
                    unlink($anexo['NMPATHANEXO']);
                }
                
                $this->titulosAReceberFactory->deleteAnexoIncompletos($anexo);
            }
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function salvaAnexoInFileServer($anexos, $cdfilial, $nrlanbolrec){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($anexos as $anexo){
                if(file_exists($anexo['NMPATHANEXO'])){
                    $arquivo = array();
                    $arquivo["b64File"] = file_get_contents($anexo['NMPATHANEXO']);
                    $arquivo["lastModified"] =  explode('_', $anexo['DSINFOANEXO'])[0];
                    $arquivo["lastModifiedDate"] = explode('_', $anexo['DSINFOANEXO'])[1];
                    $arquivo["webkitRelativePath"] = explode('_', $anexo['DSINFOANEXO'])[2];
                    $arquivo["type"] = explode('_', $anexo['DSINFOANEXO'])[3];
                    $arquivo["size"] = explode('_', $anexo['DSINFOANEXO'])[4];
                    $arquivo["name"] = explode('_', $anexo['DSINFOANEXO'])[5];
                    
                    unlink($anexo['NMPATHANEXO']);
                    
                    $anexo['NMPATHANEXO'] = $this->saveXmlInFileUplouder($arquivo);
                    
                    $anexo['CDFILIAL'] = $cdfilial;
                    $anexo['NRLANBOLREC'] = $nrlanbolrec;
                    
                    $this->titulosAReceberFactory->uptadeAnexos($anexo);
                    
                } /*else {
                    DELETAR ARQUIVO CASO ELE NAO EXISTA ?
                }*/
            }
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveXmlInFileUplouder($arquivo) {
        
        if (isset($arquivo)) {
            
            $file = array(
                "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"], 
                "b64File" => $arquivo["b64File"]);
                
            $row['ANEXOLANCBOLREC']    = array();
            $row['ANEXOLANCBOLREC'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'ANEXOLANCBOLREC', 'ANEXOLANCBOLREC');
            
            return $row['ANEXOLANCBOLREC'][0]['path'];
            
            /*
            
            $row = [
                'FILE' => [
                    [
                        "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                        "type" => $arquivo["type"], 
                        "size" => $arquivo["size"], 
                        "path" => '', 
                        "b64File" => $arquivo["b64File"]
                    ]
                ],
                'FOLDER' => 'ANEXOLANCBOLREC'
            ];
            
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */
            
        } else {
            return false;
        }
    }
    
    public function getFileServerUrl(){
        return $this->_fileServerUrl;
    }
    
    /**
     * Envia Carta de Cobrança por E-mail
     * @param $boletos
     * @param $request
     * @param $response
     * @return array
     * @throws \Exception
     */
    public function enviaCartaCobrancaPorEmail($remetente, $assunto, $mensagem, $boletos, $request, $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach($boletos as $boleto) {
                $this->enviaEmailTitulo($remetente, $boleto['DSEMAILCLIE'], $assunto, $mensagem, [], $boleto);
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Mês por Extenso
     * @param $mes
     * @return $mes
     */
    public function mesPorExtenso($mes){
        switch($mes){
            case '01':
                $mes = 'Janeiro';
                break;
            case '02':
                $mes = 'Fevereiro';
                break;
            case '03':
                $mes = 'Março';
                break;
            case '04':
                $mes = 'Abril';
                break;
            case '05':
                $mes = 'Maio';
                break;
            case '06':
                $mes = 'Junho';
                break;
            case '07':
                $mes = 'Julho';
                break;
            case '08':
                $mes = 'Agosto';
                break;
            case '09':
                $mes = 'Setembro';
                break;
            case '10':
                $mes = 'Outubro';
                break;
            case '11':
                $mes = 'Novembro';
                break;
            case '12':
                $mes = 'Dezembro';
                break;
        }
        return $mes;
    }
    
    /**
     * Formata Data
     * @param $data
     * @param $trunc
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function formataData($data, $trunc = true){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        
        if($trunc)
            $data = DateUtil::truncateData($data);

        return $data;
    }
	
	public function callWebService($url, $data){
        $header = array();
        $header[] = 'Content-Type: application/json';
        $header[] = 'Content-Length: ' . strlen(json_encode($data));
    
        $ch = curl_init();
        
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_HEADER, TRUE); 
	    curl_setopt($ch, CURLOPT_NOBODY, TRUE);
	    curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    
        $response = curl_exec($ch); 
	    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
	    $header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
	    $header = substr($response, 0, $header_size);
	    $body = substr($response, $header_size);
        
        curl_close($ch);
	    
	    return $body;
    }

}